@@ -2,7 +2,7 @@
                           rkloadagent  -  description
                              -------------------
     begin                : Sun Sep 5 2004
-    copyright            : (C) 2004, 2007, 2009 by Thomas Friedrichsmeier
+    copyright            : (C) 2004, 2007, 2009, 2011 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -28,6 +28,7 @@
 #include "../rbackend/rinterface.h"
 #include "../rkward.h"
 #include "../windows/rkworkplace.h"
+#include "../settings/rksettingsmodulegeneral.h"
 
 #include "../debug.h"
 
@@ -49,7 +50,7 @@ RKLoadAgent::RKLoadAgent (const KUrl &url, bool merge) {
 	RCommand *command;
 	
 	if (!merge) {
-		RKWardMainWindow::getMain ()->slotCloseAllEditors ();
+		RKWardMainWindow::getMain ()->slotCloseAllWindows ();
 		command = new RCommand ("remove (list=ls (all.names=TRUE))", RCommand::App | RCommand::ObjectListUpdate);
 		RKGlobals::rInterface ()->issueCommand (command);
 	}
@@ -74,7 +75,11 @@ void RKLoadAgent::rCommandDone (RCommand *command) {
 			RObjectList::getObjectList ()->setWorkspaceURL (KUrl());
 		} else {
 			RKWorkplace::mainWorkplace ()->restoreWorkplace ();
-			RKWorkplace::mainWorkplace ()->clearWorkplaceDescription ();
+			if (RKSettingsModuleGeneral::cdToWorkspaceOnLoad ()) {
+				if (RObjectList::getObjectList ()->getWorkspaceURL ().isLocalFile ()) {
+					RKGlobals::rInterface ()->issueCommand ("setwd (" + RObject::rQuote (RObjectList::getObjectList ()->getWorkspaceURL ().directory ()) + ")", RCommand::App);
+				}
+			}
 		}
 		RKWardMainWindow::getMain ()->slotSetStatusReady ();
 		RKWardMainWindow::getMain ()->setCaption (QString::null);	// trigger update of caption
