@@ -2,7 +2,7 @@
                           rksaveagent  -  description
                              -------------------
     begin                : Sun Aug 29 2004
-    copyright            : (C) 2004, 2009, 2010 by Thomas Friedrichsmeier
+    copyright            : (C) 2004, 2009, 2010, 2011 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -35,6 +35,7 @@ RKSaveAgent::RKSaveAgent (KUrl url, bool save_file_as, DoneAction when_done, KUr
 	save_url = url;
 	RKSaveAgent::when_done = when_done;
 	RKSaveAgent::load_url = load_url;
+	previous_url = RObjectList::getObjectList ()->getWorkspaceURL ();
 	save_chain = 0;
 	if (save_url.isEmpty () || save_file_as) {
 		if (!askURL ()) {
@@ -46,9 +47,9 @@ RKSaveAgent::RKSaveAgent (KUrl url, bool save_file_as, DoneAction when_done, KUr
 	RKWorkplace::mainWorkplace ()->flushAllData ();
 	save_chain = RKGlobals::rInterface ()->startChain (0);
 	
+	RObjectList::getObjectList ()->setWorkspaceURL (save_url);
 	RKWorkplace::mainWorkplace ()->saveWorkplace (save_chain);
 	RKGlobals::rInterface ()->issueCommand (new RCommand ("save.image (\"" + save_url.toLocalFile () + "\")", RCommand::App, QString::null, this), save_chain);
-	RKWorkplace::mainWorkplace ()->clearWorkplaceDescription (save_chain);
 }
 
 RKSaveAgent::~RKSaveAgent () {
@@ -70,6 +71,8 @@ bool RKSaveAgent::askURL () {
 void RKSaveAgent::rCommandDone (RCommand *command) {
 	RK_TRACE (APP);
 	if (command->hasError ()) {
+		RObjectList::getObjectList ()->setWorkspaceURL (previous_url);
+
 		int res;
 		if (when_done != DoNothing) {
 			res = KMessageBox::warningYesNoCancel (0, i18n ("Saving to file '%1' failed. What do you want to do?", save_url.path ()), i18n ("Save failed"), KGuiItem (i18n ("Try saving with a different filename")), KGuiItem (i18n ("Saving failed")));
@@ -89,8 +92,6 @@ void RKSaveAgent::rCommandDone (RCommand *command) {
 
 		// else
 		when_done = DoNothing;
-	} else {
-		RObjectList::getObjectList ()->setWorkspaceURL (save_url);
 	}
 	done ();
 }
