@@ -2,7 +2,7 @@
                           rkvariable  -  description
                              -------------------
     begin                : Thu Aug 12 2004
-    copyright            : (C) 2004, 2007, 2010 by Thomas Friedrichsmeier
+    copyright            : (C) 2004, 2007, 2010, 2011 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -117,22 +117,19 @@ is set to Unused, if _no_ cell in the row is used, Valid if _all_ cells in the r
 /** returns alignment to use for this variable */
 	CellAlign getAlignment () const;
 
-/// an empty char
-	static QString *na_char;
-/// an unknown value
-	static QString *unknown_char;
-
 /** creates/parses formatting options from the stored meta-property string. See also: getFormattingOptions () */
 	static FormattingOptions parseFormattingOptionsString (const QString &string);
 /** inverse of parseFormattingOptionsString () */
 	static QString formattingOptionsToString (const FormattingOptions& options);
+/** changes the allocated storage to contain a least length elements. More data may be allocated than acutally needed. This function only ever does upsizing. */
+	void extendToLength (int length);
 protected:
 /** Discards pending unsynced changes. */
 	void discardUnsyncedChanges ();
 /** like setNumeric, but sets chars. If internalStorage () is numeric, attempts to convert the given strings to numbers. I.e. the function behaves essentially like setText (), but operates on a range of cells. Code may assume that all data comes directly from R, is entirely valid in R. */
-	virtual void setCharacterFromR (int from_row, int to_row, QString *data);
+	virtual void setCharacterFromR (int from_row, int to_row, const QStringList &data);
 /** set numeric values in the given range. Assumes you provide enough values for the range. If internalStorage is String, all values will be converted to strings, so you should use this function only, if you know you are dealing with a numeric object. Code may assume that all data comes directly from R, is entirely valid in R. */
-	void setNumericFromR (int from_row, int to_row, double *data);
+	void setNumericFromR (int from_row, int to_row, const QVector<double> &data);
 /** reimplemented from RObject to change the internal data storage mode, if the var is being edited */
 	bool updateType (RData *new_data);
 /** Extended from RObject::EditData to actually contain data. */
@@ -170,10 +167,6 @@ protected:
 /** reimplemented from RObject */
 	void endEdit ();
 
-/** changes the allocated storage to contain a least length elements. More data may be allocated than acutally needed. This function only ever does upsizing. */
-	void extendToLength (int length);
-/** changes the allocated storage to contain a least getLength elements. More data may be allocated than acutally needed. This function only ever does downsizing. */
-	void downSize ();
 /** takes care of syncing the given range of cells */
 	void cellsChanged (int from_row, int to_row);
 /** writes the given range of cells to the backend (regardless of whether syncing should be immediate) */
