@@ -2,7 +2,7 @@
                           robjectlist  -  description
                              -------------------
     begin                : Wed Aug 18 2004
-    copyright            : (C) 2004, 2006, 2007, 2009, 2010 by Thomas Friedrichsmeier
+    copyright            : (C) 2004, 2006, 2007, 2009, 2010, 2011 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -52,7 +52,11 @@ RObjectList::RObjectList () : RContainerObject (0, QString::null) {
 	type = RObject::Workspace;
 	
 	update_chain = 0;
-	RKGlobals::tracker ()->addObject (createTopLevelEnvironment (".GlobalEnv"), this, 0);
+
+	RObject *globalenv = createTopLevelEnvironment (".GlobalEnv");
+	RKGlobals::tracker ()->beginAddObject (globalenv, this, 0);
+	childmap.insert (0, globalenv);
+	RKGlobals::tracker ()->endAddObject (globalenv, this, 0);
 }
 
 RObjectList::~RObjectList () {
@@ -168,6 +172,9 @@ void RObjectList::updateEnvironments (const QStringList &env_names, bool force_g
 		QString name = env_names[i];
 
 		RObject* obj = findChildByName (name);
+		if (obj && (i > 0) && (env_names.lastIndexOf (name, i-1) > -1)) {		// duplicate environment names can happen (e.g. if a data.frame is attached multiple times)
+			obj = 0;	// only copy the old item once
+		}
 		if (obj) {
 			// for now, we only update the .GlobalEnv. All others we assume to be static
 			if (obj->isType (GlobalEnv) && force_globalenv_update) {
@@ -175,7 +182,9 @@ void RObjectList::updateEnvironments (const QStringList &env_names, bool force_g
 			}
 		} else {
 			obj = createTopLevelEnvironment (name);
-			RKGlobals::tracker ()->addObject (obj, this, i);
+			RKGlobals::tracker ()->beginAddObject (obj, this, i);
+			childmap.insert (i, obj);
+			RKGlobals::tracker ()->endAddObject (obj, this, i);
 		}
 		newchildmap.insert (i, obj);
 	}
@@ -206,63 +215,33 @@ REnvironmentObject *RObjectList::createTopLevelEnvironment (const QString &name)
 	return envobj;
 }
 
-RObject *RObjectList::findObject (const QString &name, bool is_canonified) const {
+RObject *RObjectList::findObjects (const QStringList &path, RObjectSearchMap *matches, const QString &op) {
 	RK_TRACE (OBJECTS);
+	RK_ASSERT (op == "$");
 
-	QString canonified = name;
-	if (!is_canonified) canonified = canonifyName (name);
-
-	// TODO: there could be objects with "::" in their names!
-	if (canonified.contains ("::")) {
-		QString env = canonified.section ("::", 0, 0);
-		QString remainder = canonified.section ("::", 1);
-
-		RObject *found = findChildByNamespace (env);
-		if (!found) return 0;
-
-		return (found->findObject (remainder, true));
-	} else if (canonified.startsWith (".GlobalEnv$")) {
-		return (getGlobalEnv ()->findObject (canonified.mid (11), true));
-	} else if (canonified == ".GlobalEnv") {
-		return (getGlobalEnv ());
-	}
-
-	// no "::"-qualification given, do normal search in all environments, return first match
-	for (int i = 0; i < childmap.size (); ++i) {
-		RObject *found = childmap[i]->findObject (canonified, true);
-		if (found) return found;
-	}
-	return 0;
-}
-
-void RObjectList::findObjectsMatching (const QString &partial_name, RObjectSearchMap *current_list, bool name_is_canonified) const {
-	RK_TRACE (OBJECTS);
-	RK_ASSERT (current_list);
-
-	QString canonified = partial_name;
-	if (!name_is_canonified) canonified = canonifyName (partial_name);
-
-	// TODO: there could be objects with "::" in their names!
-	if (canonified.contains ("::")) {
-		QString env = canonified.section ("::", 0, 0);
-		QString remainder = canonified.section ("::", 1);
-
-		RObject *found = findChildByNamespace (env);
-		if (!found) return;
+	if (path.value (1) == "::") {
+		RObject *environment = findChildByNamespace (path[0]);
+		if (!environment) return 0;
 		
-		found->findObjectsMatching (remainder, current_list, true);
-		return;
-	} else if (canonified.startsWith (".GlobalEnv$")) {
-		getGlobalEnv ()->findObjectsMatching (canonified.mid (11), current_list, true);
-		return;
-	} else if (canonified == ".GlobalEnv") {
-		current_list->insert (canonified, getGlobalEnv());	// but do not return, there will be at least one further match in base
+		return environment->findObjects (path.mid (2), matches, "$");
+	} else if (path.value (1) == ":::") {
+		RObject *environment = findChildByNamespace (path[0]);
+		if (environment) environment = static_cast<REnvironmentObject*> (environment)->namespaceEnvironment ();
+		if (!environment) return 0;
+
+		return environment->findObjects (path.mid (2), matches, "$");
+	} else if (path.value (0) == ".GlobalEnv") {
+		if (path.length () > 1) return getGlobalEnv ()->findObjects (path.mid (2), matches, "$");
+		else if (matches) matches->insert (path.value (0), getGlobalEnv ());	// no return, here: At least one more match will be found in base
+		else return getGlobalEnv ();
 	}
 
 	// no namespace given. Search all environments for matches
 	for (int i = 0; i < childmap.size (); ++i) {
-		childmap[i]->findObjectsMatching (canonified, current_list, true);
+		RObject *found = childmap[i]->findObjects (path, matches, "$");
+		if (found && !matches) return found;
 	}
+	return 0;
 }
 
 REnvironmentObject* RObjectList::findChildByNamespace (const QString &namespacename) const {
@@ -272,7 +251,7 @@ REnvironmentObject* RObjectList::findChildByNamespace (const QString &namespacen
 		RObject* child = childmap[i];
 		RK_ASSERT (child->isType (Environment));
 		REnvironmentObject* env = static_cast<REnvironmentObject *> (child);
-		if (env->namespaceName () == namespacename) {
+		if (env->packageName () == namespacename) {
 			return env;
 		}
 	}
@@ -312,7 +291,7 @@ REnvironmentObject *RObjectList::getGlobalEnv () {
 	RObjectList *list = getObjectList ();
 	RK_ASSERT (list);
 
-	RK_ASSERT (list->numChildren ());
+	RK_ASSERT (!list->isEmpty ());
 	REnvironmentObject *envobj = static_cast<REnvironmentObject*> (list->childmap[0]);
 	RK_ASSERT (envobj);
 	RK_ASSERT (envobj->isType (RObject::GlobalEnv));
