@@ -2,7 +2,7 @@
                           rkstandardicons  -  description
                              -------------------
     begin                : Wed Oct 24 2007
-    copyright            : (C) 2007, 2009, 2010 by Thomas Friedrichsmeier
+    copyright            : (C) 2007, 2009, 2010, 2011 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -52,6 +52,7 @@ public:
 		ActionPasteInsideTable,
 		ActionPasteInsideSelection,
 
+		ActionDelete,
 		ActionAddRight,
 		ActionRemoveLeft,
 
@@ -70,6 +71,7 @@ public:
 		ActionWindowDuplicate,
 
 		ActionClear,
+		ActionInterrupt,
 
 		ActionDetachWindow,
 		ActionAttachWindow,
@@ -86,7 +88,7 @@ public:
 		ObjectDataLogical,
 		ObjectDataUnknown,
 		ObjectDataOther,
-		ObjectObjectList,
+		ObjectPseudo,
 
 		WindowDataFrameEditor,
 		WindowCommandEditor,
