@@ -15,5 +15,10 @@ function calculate () {
 
 function printout () {
 	echo ('rk.header ("Goodness of Fit for Rasch Models (' + getValue("x") + ')")\n');
-	echo ('rk.print (GoFRasch.res)\n');
+	echo ('rk.print ("Call:")\n');
+	echo ('rk.print.literal (deparse(GoFRasch.res$call, width.cutoff=500))\n');
+	echo ('rk.header ("Parametric Bootstrap test:", level=4)\n');
+	echo ('rk.print (paste("Chi-squared statistic:", round(GoFRasch.res$Tobs, digits=3)))\n');
+	echo ('rk.print (paste("Bootstrap samples:", GoFRasch.res$B))\n');
+	echo ('rk.print (paste("p-value:", GoFRasch.res$p.value))\n');
 }
