@@ -19,12 +19,11 @@
 #include "rcommandreceiver.h"
 #include "rinterface.h"
 #include "../windows/rkcommandlog.h"
+#include "rkrbackendprotocol_shared.h"
 
 #include "../debug.h"
 #include "../rkglobals.h"
 
-#define MAX_RECEIVERS 3
-
 RCommandBase::RCommandBase (bool is_chain) {
 	is_command_chain = is_chain;
 	RCommandBase::parent = 0;
@@ -56,11 +55,13 @@ RCommand::RCommand(const QString &command, int type, const QString &rk_equiv, RC
 	else _command = command;
 	if (_command.isEmpty ()) _type |= EmptyCommand;
 	status = 0;
+	has_been_run_up_to = 0;
 	_rk_equiv = rk_equiv;
-	RCommand::receivers = new RCommandReceiver* [MAX_RECEIVERS];
-	num_receivers = 0;
-	addReceiver (receiver);
-	addReceiver (RKCommandLog::getLog ());
+	for (int i = 0; i < MAX_RECEIVERS_PER_RCOMMAND; ++i) receivers[i] = 0;
+	if (!(type & Internal)) {
+		addReceiver (receiver);
+		addReceiver (RKCommandLog::getLog ());
+	}
 }
 
 RCommand::~RCommand(){
@@ -69,7 +70,6 @@ RCommand::~RCommand(){
 	for (QList<ROutput*>::const_iterator it = output_list.constBegin (); it != output_list.constEnd (); ++it) {
 		delete (*it);
 	}
-	delete [] receivers;
 	// The output_list itself is cleared automatically
 }
 
@@ -78,13 +78,15 @@ void RCommand::addReceiver (RCommandReceiver *receiver) {
 
 	if (!receiver) return;
 
-	if (num_receivers >= MAX_RECEIVERS) {
-		RK_DO (qDebug ("Too many receivers for command"), RBACKEND, DL_ERROR);
-		return;
+	for (int i = 0; i < MAX_RECEIVERS_PER_RCOMMAND; ++i) {
+		if (receivers[i] == 0) {
+			receivers[i] = receiver;
+			receiver->addCommand (this);
+			return;
+		}
 	}
 
-	receivers[num_receivers++] = receiver;
-	receiver->addCommand (this);
+	RK_DO (qDebug ("Too many receivers for command"), RBACKEND, DL_ERROR);
 }
 
 void RCommand::removeReceiver (RCommandReceiver *receiver) {
@@ -92,27 +94,21 @@ void RCommand::removeReceiver (RCommandReceiver *receiver) {
 
 	if (!receiver) return;
 
-	RCommandReceiver **newlist = new RCommandReceiver* [MAX_RECEIVERS];
-	int num_new_receivers = 0;
-	for (int i=0; i < num_receivers; ++i) {
-		if (receivers[i] != receiver) {
-			newlist[num_new_receivers++] = receivers[i];
+	for (int i = 0; i < MAX_RECEIVERS_PER_RCOMMAND; ++i) {
+		if (receivers[i] == receiver) {
+			receivers[i] = 0;
+			return;
 		}
 	}
 
-	if (num_new_receivers == num_receivers) {
-		RK_DO (qDebug ("Was not a receiver in RCommand::removeReceiver"), RBACKEND, DL_WARNING);
-	}
-
-	delete [] receivers;
-	receivers = newlist;
-	num_receivers = num_new_receivers;
+	RK_DO (qDebug ("Was not a receiver in RCommand::removeReceiver: %p", receiver), RBACKEND, DL_WARNING);
 }
 
 void RCommand::finished () {
 	RK_TRACE (RBACKEND);
 
-	for (int i=0; i < num_receivers; ++i) {
+	for (int i=0; i < MAX_RECEIVERS_PER_RCOMMAND; ++i) {
+		if (receivers[i] == 0) continue;
 		receivers[i]->delCommand (this);
 		receivers[i]->rCommandDone (this);
 	}
@@ -121,12 +117,23 @@ void RCommand::finished () {
 void RCommand::newOutput (ROutput *output) {
 	RK_TRACE (RBACKEND);
 
-	for (int i=0; i < num_receivers; ++i) {
+	for (int i=0; i < MAX_RECEIVERS_PER_RCOMMAND; ++i) {
+		if (receivers[i] == 0) continue;
 		receivers[i]->newOutput (this, output);
 	}
 }
 
-QString RCommand::error () {
+void RCommand::commandLineIn () {
+	RK_TRACE (RBACKEND);
+	RK_ASSERT (_type & User);
+
+	for (int i=0; i < MAX_RECEIVERS_PER_RCOMMAND; ++i) {
+		if (receivers[i] == 0) continue;
+		receivers[i]->userCommandLineIn (this);
+	}
+}
+
+QString RCommand::error () const {
 	RK_TRACE (RBACKEND);
 
 	QString ret;
@@ -138,7 +145,7 @@ QString RCommand::error () {
 	return ret;
 }
 
-QString RCommand::output () {
+QString RCommand::output () const {
 	RK_TRACE (RBACKEND);
 
 	QString ret;
@@ -150,7 +157,7 @@ QString RCommand::output () {
 	return ret;
 }
 
-QString RCommand::warnings () {
+QString RCommand::warnings () const {
 	RK_TRACE (RBACKEND);
 
 	QString ret;
@@ -162,7 +169,7 @@ QString RCommand::warnings () {
 	return ret;
 }
 
-QString RCommand::fullOutput () {
+QString RCommand::fullOutput () const {
 	RK_TRACE (RBACKEND);
 
 	QString ret;
@@ -171,3 +178,36 @@ QString RCommand::fullOutput () {
 	}
 	return ret;
 }
+
+QString RCommand::remainingCommand () const {
+	RK_TRACE (RBACKEND);
+	RK_ASSERT (_type & User);	// not a grave problem, if it's not, but not useful, either
+
+	return _command.mid (has_been_run_up_to);
+}
+
+void RCommand::mergeAndDeleteProxy (RCommandProxy *proxy) {
+	RK_TRACE (RBACKEND);
+
+	RK_ASSERT (proxy);
+	RK_ASSERT (proxy->id == _id);
+	RK_ASSERT (proxy->type == _type);
+
+	status = proxy->status;
+	has_been_run_up_to = proxy->has_been_run_up_to;
+	swallowData (*proxy);
+	delete proxy;
+}
+
+RCommandProxy* RCommand::makeProxy () const {
+	RK_TRACE (RBACKEND);
+	RK_ASSERT (status == 0);	// Initialization from an already touched command is not a real problem, but certainly no expected usage
+	RK_ASSERT (has_been_run_up_to == 0);
+	RK_ASSERT (getDataType () == RData::NoData);
+
+	RCommandProxy *ret = new RCommandProxy (_command, _type);
+	ret->id = _id,
+	ret->status = status;
+	ret->has_been_run_up_to = has_been_run_up_to;
+	return ret;
+}
