@@ -2,7 +2,7 @@
                           rcommand.h  -  description
                              -------------------
     begin                : Mon Nov 11 2002
-    copyright            : (C) 2002, 2006, 2007, 2009 by Thomas Friedrichsmeier
+    copyright            : (C) 2002, 2006, 2007, 2009, 2010 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -25,8 +25,11 @@
 
 #include "rdata.h"
 
+#define MAX_RECEIVERS_PER_RCOMMAND 3
+
 class RCommandReceiver;
 class RCommand;
+class RCommandProxy;
 class RCommandChain;
 
 /** Base class for RCommand and RCommandChain, to make it possible to store both in the same list */
@@ -50,6 +53,8 @@ private:
 @see RInterface::startChain
 @see RInterface::closeChain */
 class RCommandChain : public RCommandBase {
+public:
+	bool isClosed () const { return closed; };
 protected:
 friend class RCommandStack;
 friend class RCommandStackModel;
@@ -63,6 +68,7 @@ friend class RCommandStackModel;
 QString is, that additionally we store information on whether the output was "normal", "warning", or an "error". */
 struct ROutput {
 	enum ROutputType {
+		NoOutput,		/**< No output. Rarely used. */
 		Output,			/**< normal output */
 		Warning,		/**< R warning */
 		Error			/**< R error */
@@ -73,24 +79,6 @@ struct ROutput {
 
 typedef QList<ROutput*> ROutputList;
 
-/*
-struct RGetValueRequest {
-private:
-friend class RInterface;
-friend class RThread;
-	char **call;
-	int call_length;
-};
-
-struct RGetValueReply {
-private:
-friend class RInterface;
-friend class RThread;
-	char **reply;
-	int reply_length;
-};
-*/
-
 /** For introductory information on using RCommand, see \ref UsingTheInterfaceToR 
 
 	This class is used to encapsulate an R-command, so it can be easily identified
@@ -110,7 +98,6 @@ friend class RThread;
 	kept around very long, so they should not be a memory issue.
   *@author Thomas Friedrichsmeier
   */
-  
 class RCommand : public RData, public RCommandBase {
 public:
 /** constructs an RCommand.
@@ -120,46 +107,50 @@ public:
 @param receiver The RCommandReceiver this command should be passed on to, when finished.
 @param flags A freely assignable integer, that you can use to identify what the command was all about. Only the RCommandReceiver handling the results will have to know what exactly the flags mean.
 */
-	explicit RCommand (const QString &command, int type = 0, const QString &rk_equiv = QString::null, RCommandReceiver *receiver=0, int flags=0);
+	explicit RCommand (const QString &command, int type, const QString &rk_equiv = QString::null, RCommandReceiver *receiver=0, int flags=0);
 /** destructor. Note: you should not delete RCommands manually. This is done in RInterface. TODO: make protected */
 	~RCommand();
 /** @returns the type as specified in RCommand::RCommand */
-	int type () { return _type; };
+	int type () const { return _type; };
 /** @returns the raw command status. @see CommandStatus */
-	int getStatus () { return status; };
+	int getStatus () const { return status; };
 /** @returns the rk_equiv as specified in RCommand::RCommand */
-	QString rkEquivalent () { return _rk_equiv; };
+	QString rkEquivalent () const { return _rk_equiv; };
 /** @returns the command string (i.e. the input) as specified in RCommand::RCommand */
-	QString command () { return _command; };
+	QString command () const { return _command; };
+/** @returns like command(), but for user commands, which have been run, partially, returns only the remaining portion of the command. */
+	QString remainingCommand () const;
 /** Each RCommand is assigned a unique integer id (incrementing from 0 to integer overflow) upon creation. This returns this id. 
 	@returns the unique id of this command */
-	int id () { return _id; };
+	int id () const { return _id; };
 /* TODO: Adjust these two functions to allow re-getting of output and error-messages from logs */
 /** @returns the full output of the command, i.e. all "regular" output, warning messages, and errors, in the order they were encountered. @see RCommand::output @see RCommand::error @see RCommand::warnings */
-	QString fullOutput ();
+	QString fullOutput () const;
 /** @returns the "regular" (ROutput::Output) output of the command, if any (e.g. "[1] 1" for "print (1)"). @see RCommand::succeeded @see RCommand::hasOutput */
-	QString output ();
+	QString output () const;
 /** @returns the warning message(s) given by R, if any. @see RCommand::output @see RCommand::error */
-	QString warnings ();
+	QString warnings () const;
 /** @returns the error message given by R, if any. @see RCommand::failed @see RCommand::hasError */
-	QString error ();
+	QString error () const;
 /** Types of commands (potentially more to come), bitwise or-able,
 	although partially exclusive. See \ref UsingTheInterfaceToR for a overview of what these are used for. TODO: find out, why Canceled is in here, and document that fact. */
 	enum CommandTypes {
 		User=1,		/**< Command was created directly by the user (e.g. in the console or in a command editor window) */
-		Plugin=2,		/**< Command comes from a plugin */
-		PluginCom=4,	/**< Command comes from a plugin, and is used by the plugin to communicate directly with R (no real use so far) */
-		App=8,			/**< Command comes from the application (e.g. loading / saving the workspace */
-		Sync=16,		/**< Command is used to sync data to or from R-space. Typically used in the editor classes */
-		EmptyCommand=32,		/**< Command is empty and will not be processed (an empty command may be used as a "marker") */
-		Console=64,	/**< Command originated in the console. These commands will get some extra treatment in RKwatch */
-		GetIntVector=512,			/**< Try to fetch result as an array of integers */
-		GetStringVector=1024,	/**< Try to fetch result as an array of chars */
-		GetRealVector=2048,		/**< Try to fetch result as an array of doubles */
-		GetStructuredData=4096,		/**< Try to fetch result as an RData structure */
-		DirectToOutput=8192,		/**< Append command output to the HTML-output file */
-		ObjectListUpdate=16384,		/**< The command may change the list of objects available. Do an update */
-		QuitCommand=32768		/**< The R backend should be killed */
+		Plugin=1 << 1,		/**< Command comes from a plugin */
+		App=1 << 2,			/**< Command comes from the application (e.g. loading / saving the workspace */
+		Sync=1 << 3,		/**< Command is used to sync data to or from R-space. Typically used in the editor classes */
+		EmptyCommand=1 << 4,		/**< Command is empty and will not be processed (an empty command may be used as a "marker") */
+		Console=1 << 5,	/**< Command originated in the console. These commands will get some extra treatment in RKwatch */
+		Internal=1 << 6,	/**< Command is meant to be used in the backend, only. Do not use outside rbackend classes! */
+		Silent=1 << 7,	/**< Command can be interrupted, but is otherwise an internal command. In particular it should not be carbon copied to the output. */
+		GetIntVector=1 << 8,			/**< Try to fetch result as an array of integers */
+		GetStringVector=1 << 9,	/**< Try to fetch result as an array of chars */
+		GetRealVector=1 << 10,		/**< Try to fetch result as an array of doubles */
+		GetStructuredData=1 << 11,		/**< Try to fetch result as an RData structure */
+		CCOutput=1 << 12,		/**< Append command output to the HTML-output file */
+		CCCommand=1 << 13,		/**< Append the command itself to the HTML-output file */
+		ObjectListUpdate=1 << 14,		/**< The command may change the list of objects available. Do an update */
+		QuitCommand=1 << 15		/**< The R backend should be killed */
 	};
 	enum CommandStatus {
 		Running=1,						/**< command is currently running */
@@ -174,25 +165,25 @@ public:
 		Canceled=8192				/**< Command was cancelled. */
 	};
 /** the command has been passed to the backend. */
-	bool wasTried () { return (status & WasTried); };
+	bool wasTried () const { return (status & WasTried); };
 /** the command failed */
-	bool failed () { return (status & Failed); };
+	bool failed () const { return (status & Failed); };
 /** the command was cancelled before it was executed */
-	bool wasCanceled () { return (wasTried () && failed () && (status & Canceled)); }
+	bool wasCanceled () const { return (wasTried () && failed () && (status & Canceled)); }
 /** the command succeeded (wasTried () && (!failed ()) */
-	bool succeeded () { return ((status & WasTried) && !(status & Failed)); };
+	bool succeeded () const { return ((status & WasTried) && !(status & Failed)); };
 /** command has a string output retrievable via RCommand::output () */
-	bool hasOutput () { return (status & HasOutput); };
+	bool hasOutput () const { return (status & HasOutput); };
 /** command has a string output retrievable via RCommand::warnings () */
-	bool hasWarnings () { return (status & HasWarnings); };
+	bool hasWarnings () const { return (status & HasWarnings); };
 /** command has an error-message retrievable via RCommand::error () */
-	bool hasError () { return (status & HasError); };
+	bool hasError () const { return (status & HasError); };
 /** backend rejected command as being incomplete */
-	bool errorIncomplete () { return (status & ErrorIncomplete); };
+	bool errorIncomplete () const { return (status & ErrorIncomplete); };
 /** backend rejected command as having a syntax error */
-	bool errorSyntax () { return (status & ErrorSyntax); };
+	bool errorSyntax () const { return (status & ErrorSyntax); };
 /** return the flags associated with the command. Those are the same that you specified in the constructor, RKWard does not touch them. @see RCommand::RCommand */
-	int getFlags () { return (_flags); };
+	int getFlags () const { return (_flags); };
 /** Add an additional listener to the command */
 	void addReceiver (RCommandReceiver *receiver);
 /** Remove a receiver from the list. This may be needed when a listener wants to self-destruct, to make sure we don't try to send any further info there */
@@ -201,8 +192,11 @@ public:
 	ROutputList &getOutput () { return output_list; };
 /** modify the command string. DO NOT CALL THIS after the command has been submitted! */
 	void setCommand (const QString &command) { _command = command; };
+
+/** creates a proxy for this RCommand */
+	RCommandProxy* makeProxy () const;
+	void mergeAndDeleteProxy (RCommandProxy *proxy);
 private:
-friend class RThread;
 friend class RInterface;
 friend class RCommandStack;
 friend class RCommandStackModel;
@@ -210,16 +204,18 @@ friend class RCommandStackModel;
 	void finished ();
 /** new output was generated. Pass on to receiver(s) */
 	void newOutput (ROutput *output);
+/** next line of command has been transmitted. Pass on to receiver(s). Only called for RCommand::User type commands */
+	void commandLineIn ();
 	ROutputList output_list;
 	QString _command;
 	int _type;
 	int _flags;
 	int status;
+	int has_been_run_up_to;
 	QString _rk_equiv;
 	int _id;
 	static int next_id;
-	int num_receivers;
-	RCommandReceiver **receivers;
+	RCommandReceiver *receivers[MAX_RECEIVERS_PER_RCOMMAND];
 };
 
 #endif
