@@ -7,10 +7,11 @@
 	eval (substitute (attr (x, ".rk.meta") <<- m))
 }
 
-".rk.set.invalid.field" <- function (x, r, d) {
+".rk.set.invalid.field" <- function (x, row, value) {
 	l <- attr (x, ".rk.invalid.fields");
 	if (is.null (l)) l <- list ();
-	l[[as.character(r)]] <- d;
+	l[[as.character(row)]] <- value;
+	if (length (l) == 0) l <- NULL
 	eval (substitute (attr (x, ".rk.invalid.fields") <<- l))
 }
 
@@ -32,23 +33,25 @@
 	for (i in 1:dim (x)[2]) {
 		attriblist[[names (x)[i]]] <- attributes (x[[i]])
 	}
-	eval (substitute (x <<- x[-index,]))
-	eval (substitute (row.names (x) <<- c (1:dim(x)[1])))
-	for (i in 1:dim (x)[2]) {
-		eval (substitute (attributes (x[[i]]) <<- attriblist[[names (x)[i]]]))
+	newx <- x[-index, , drop=FALSE]
+	row.names (newx) <- c (1:dim(newx)[1])
+	for (i in 1:dim (newx)[2]) {
+		attributes (newx[[i]]) <- attriblist[[names (newx)[i]]]
 	}
+	eval (substitute (x <<- newx))
 }
 
 # function below is only needed to ensure a nice ordering of the columns. Simply adding a new column would be much easier than this.
 ".rk.data.frame.insert.column" <- function (x, label, index=0) {
+	column <- rep (as.numeric (NA), times=dim (x)[1])
 	if ((index == 0) || (index > dim (x)[2])) {	# insert column at end
-		eval (substitute (x[[label]] <<- c (NA)))
+		eval (substitute (x[[label]] <<- column))
 	} else {
 		for (i in dim (x)[2]:index) {
 			eval (substitute (x[i+1] <<- x[[i]]))
 			eval (substitute (names (x)[i+1] <<- names (x)[i]))
 		}
-		eval (substitute (x[index] <<- c (NA)))
+		eval (substitute (x[index] <<- column))
 		eval (substitute (names (x)[index] <<- label))
 	}
 }
@@ -60,13 +63,17 @@
 	}
 }
 
+".rk.rkreply" <- NULL
+".rk.set.reply" <- function (x) .rk.rkreply <<- x
+
 ".rk.do.call" <- function (x, args=NULL) {
+	.rk.set.reply (NULL)
 	.Call ("rk.do.command", c (x, args));
-	if (exists (".rk.rkreply")) {
-		return (.rk.rkreply)
-	} else {
-		return (NULL)
-	}
+	return (.rk.rkreply)
+}
+
+".rk.do.plain.call" <- function (x, args=NULL, synchronous=TRUE) {
+	.Call ("rk.do.generic.request", c (x, args), isTRUE (synchronous))
 }
 
 # package information formats may - according to the help - be subject to change. Hence this function to cope with "missing" values
@@ -129,7 +136,8 @@
 
 # overriding q, to ask via GUI instead. Arguments are not interpreted.
 "q" <- function (save = "default", status = 0, runLast = TRUE, ...) {
-	.rk.do.call ("quit")
+	res <- .rk.do.plain.call ("quit")
+	if (length (res) && (res == "FALSE")) stop ("Quitting was cancelled")
 }
 
 "quit" <- function (save = "default", status = 0, runLast = TRUE, ...) {
@@ -145,13 +153,14 @@
 ".rk.watched.symbols" <- new.env ()
 
 # override makeActiveBinding: If active bindings are created in globalenv (), watch them properly
+.rk.makeActiveBinding.default <- base::makeActiveBinding
 "makeActiveBinding" <- function (sym, fun, env, ...) {
 	if (identical (env, globalenv ())) {
-		base::makeActiveBinding (sym, fun, .rk.watched.symbols, ...)
+		.rk.makeActiveBinding.default (sym, fun, .rk.watched.symbols, ...)
 		f <- .rk.make.watch.f (sym)
-		base::makeActiveBinding (sym, f, globalenv (), ...)
+		.rk.makeActiveBinding.default (sym, f, globalenv (), ...)
 	} else {
-		base::makeActiveBinding (sym, fun, env, ...)
+		.rk.makeActiveBinding.default (sym, fun, env, ...)
 	}
 }
 
@@ -183,7 +192,7 @@
 	#assign (k, get (k, envir=globalenv ()), envir=.rk.watched.symbols)
 	rm (list=k, envir=globalenv ())
 
-	base::makeActiveBinding (k, f, globalenv ())
+	.rk.makeActiveBinding.default (k, f, globalenv ())
 
 	invisible (TRUE)
 }
@@ -245,7 +254,7 @@
 	eval (substitute (x <- y), envir=envir)
 }
 
-".rk.get.structure.new" <- function (x, name, envlevel=0, namespacename=NULL) {
+".rk.get.structure" <- function (x, name, envlevel=0, namespacename=NULL) {
 	.Call ("rk.get.structure", x, as.character (name), as.integer (envlevel), namespacename)
 }
 
@@ -253,20 +262,15 @@
 	tryCatch (asNamespace (name), error = function(e) NULL)
 }
 
-".rk.get.structure" <- .rk.get.structure.new
-
 ".rk.get.structure.global" <- function (name, envlevel=0, namespacename=NULL) {
 	.Call ("rk.get.structure.global", as.character (name), as.integer (envlevel), namespacename)
 }
 
-".rk.get.formals" <- function (x) 
-{
-    f <- formals (x)
-    r <- as.character(lapply(f, function(v) {
-        if (is.character(v)) return(encodeString(v, quote = "\"")) else return(v)
-    }))
-    names (r) <- names (f)
-    r
+".rk.get.slots" <- function (x) {
+	slotnames <- methods::slotNames (class (x))
+	ret <- lapply (slotnames, function (slotname) slot (x, slotname))
+	names (ret) <- slotnames
+	ret
 }
 
 ".rk.get.environment.children" <- function (x, envlevel=0, namespacename=NULL) {
@@ -294,16 +298,10 @@
 
 ".rk.output.html.file" <- NULL
 
-".rk.rkreply" <- NULL
-
-".rk.set.reply" <- function (x) .rk.rkreply <<- x
-
 "Sys.setlocale" <- function (category = "LC_ALL", locale = "", ...) {
 	if (category == "LC_ALL" || category == "LC_CTYPE" || category == "LANG") {
-		.rk.do.call ("preLocaleChange", NULL);
-		if (!is.null (.rk.rkreply)) {
-			if (.rk.rkreply == FALSE) stop ("Changing the locale was cancelled by user");
-		}
+		allow <- .rk.do.plain.call ("preLocaleChange", NULL)
+		if (length (allow) && (allow == "FALSE")) stop ("Changing the locale was cancelled by user")
 
 		ret <- base::Sys.setlocale (category, locale, ...)
 
@@ -315,8 +313,9 @@
 }
 
 "setwd" <- function () {
-	eval (body (base::setwd))
-	invisible (.rk.do.call ("wdChange", NULL));
+	ret <- eval (body (base::setwd))
+	.rk.do.plain.call ("wdChange", base::getwd (), synchronous=FALSE)
+	invisible (ret)
 }
 formals (setwd) <- formals (base::setwd)
 
@@ -342,9 +341,9 @@ formals (setwd) <- formals (base::setwd)
 "rk.record.commands" <- function (filename, include.sync.commands = FALSE) {
 	if (is.null (filename)) filename = ""
 
-	res <- .rk.do.call ("recordCommands", c(as.character (filename), if (include.sync.commands) "include.sync" else "normal"))
+	res <- .rk.do.plain.call ("recordCommands", c(as.character (filename), if (include.sync.commands) "include.sync" else "normal"))
 
-	if (is.null (res)) invisible (TRUE)
+	if (!length (res)) invisible (TRUE)
 	else {
 		warning (res)
 		invisible (FALSE)
@@ -386,39 +385,112 @@ formals (setwd) <- formals (base::setwd)
 	invisible (TRUE)
 }
 
-"select.list" <- function () {
-	# the "list" parameter was renamed to "choices" in R 2.11.0
-	if (!exists ("list", inherits=FALSE)) list <- choices
-	# the "graphics" parameter was introduced in R 2.11.0, so we cannot rely on its existance
-	if (!exists ("graphics", inherits=FALSE)) graphics <- TRUE
-	if (graphics) {
-		return (rk.select.list (list, preselect, multiple, title))
-	}
+# Tries to replace a function inside its environemnt/namespace.
+# Function formals are copied from the original.
+# A backup of the original is stored as rkward::.rk.FUNCTIONNAME.default
+"rk.replace.function" <- function (functionname, environment, replacement, copy.formals=TRUE) {
+	original <- get (functionname, envir=environment, inherits=FALSE)
 
-	# for text list, use the default implementation
-	eval (body (.rk.select.list.default))
+	# create a backup
+	backupname <- paste (".rk.", functionname, ".default", sep="")
+	assign (backupname, original, envir=as.environment ("package:rkward"))
+
+	if (copy.formals) formals (replacement) <- formals (original)
+	environment (replacement) <- environment (original)
+	assign (functionname, replacement, envir=as.environment ("package:rkward"))
+	try (
+		if (bindingIsLocked (functionname, environment)) {
+			unlockBinding (functionname, environment)
+			on.exit (lockBinding (functionname, environment))
+		}
+	)
+	try (
+		if (isNamespace (environment)) {
+			assignInNamespace (functionname, replacement, ns=environment)
+		} else {
+			assignInNamespace (functionname, replacement, envir=environment)
+		}
+	)
+	try (
+		assign (functionname, replacement, envir=environment)
+	)
+
+	invisible (NULL)
 }
-formals (select.list) <- formals (utils::select.list)
-.rk.select.list.default <- utils::select.list
 
-"menu" <- function () {
-	if (graphics) {
-		res <- rk.select.list (choices, multiple=FALSE, title=title)
-		return (match(res, choices, nomatch = 0L))
-	}
-
-	# for text menus, use the default implementation
-	eval (body (.rk.menu.default))
-}
-formals (menu) <- formals (utils::menu)
-.rk.menu.default <- utils::menu
-
-# where masking is not enough, we need to assign in the namespace. This can only be done after package loading,
+# where masking is not enough, we need to assign in the original environment / namespace. This can only be done after package loading,
 # so we have a separate function for that.
 ".rk.fix.assignments" <- function () {
-	assignInNamespace ("menu", menu, envir=as.environment ("package:utils"))
-	assignInNamespace ("select.list", select.list, envir=as.environment ("package:utils"))
-	
+	## History manipulation function (overloads for functions by the same name in package utils)
+	rk.replace.function ("loadhistory",  as.environment ("package:utils"),
+		function (file = ".Rhistory") {
+			invisible (.rk.do.plain.call ("commandHistory", c ("set", readLines (file))))
+		}, copy.formals = FALSE)
+
+	rk.replace.function ("savehistory",  as.environment ("package:utils"),
+		function (file = ".Rhistory") {
+			invisible (writeLines (.rk.do.plain.call ("commandHistory", "get"), file))
+		}, copy.formals = FALSE)
+
+	rk.replace.function ("timestamp",  as.environment ("package:utils"),
+		function (stamp = date(), prefix = "##------ ", suffix = " ------##", quiet = FALSE) {
+			stamp <- paste(prefix, stamp, suffix, sep = "")
+			.rk.do.plain.call ("commandHistory", c ("append", stamp))
+			if (!quiet) cat(stamp, sep = "\n")
+			invisible(stamp)
+		}, copy.formals = FALSE)
+
+	## Interactive menus
+	rk.replace.function ("select.list", as.environment ("package:utils"), 
+		function () {
+			# the "list" parameter was renamed to "choices" in R 2.11.0
+			if (!exists ("list", inherits=FALSE)) list <- choices
+			# the "graphics" parameter was introduced in R 2.11.0, so we cannot rely on its existance
+			if (!exists ("graphics", inherits=FALSE)) graphics <- TRUE
+			if (graphics) {
+				return (rk.select.list (list, preselect, multiple, title))
+			}
+
+			# for text list, use the default implementation
+			eval (body (rkward::.rk.select.list.default))
+		})
+
+	rk.replace.function ("menu", as.environment ("package:utils"),
+		function () {
+			if (graphics) {
+				res <- rk.select.list (choices, multiple=FALSE, title=title)
+				return (match(res, choices, nomatch = 0L))
+			}
+
+			# for text menus, use the default implementation
+			eval (body (.rk.menu.default))
+		})
+
+	## Add output synchronisation across system(), and system2() calls.
+	rk.replace.function ("system", base::.BaseNamespaceEnv,
+		function () {
+			if (!exists ("ignore.stdout", inherits=FALSE)) ignore.stdout <- FALSE	# ignore.stdout was introduced in R 2.12.0
+
+			if (!(intern || (ignore.stdout && ignore.stderr))) {
+				.Call ("rk.sync.output", 0)
+				on.exit (.Call ("rk.sync.output", 1), TRUE)
+			}
+
+			eval (body (.rk.system.default))
+		})
+
+	# NOTE: system2 was not introduced before R 2.12.0 (or was it 2.11.0?)
+	if (exists ("system2", base::.BaseNamespaceEnv)) {
+		rk.replace.function ("system2", base::.BaseNamespaceEnv,
+			function () {
+				if (stdout != "" || stderr != "") {
+					.Call ("rk.sync.output", 0)
+					on.exit (.Call ("rk.sync.output", 1), TRUE)
+				}
+				eval (body (.rk.system2.default))
+			})
+	}
+
 	# call separate assignments functions:
 	if (exists (".rk.fix.assignments.graphics")) eval (body (.rk.fix.assignments.graphics)) # internal_graphics.R
 }
