@@ -74,6 +74,13 @@
 	}
 }
 
+# Drop-in replacement for list(). Returns a list of the given arguments, but with names set according to rk.get.description
+"rk.list" <- function (...) {
+	ret <- list (...)
+	names (ret) <- rk.get.description (...)
+	ret
+}
+
 # this is basically copied from R-base table (). Returns the arguments passed to ... as a character vector
 "rk.list.names" <- function(..., deparse.level=2) {
 	l <- as.list(substitute(list(...)))[-1]
@@ -107,7 +114,13 @@
 }
 
 "rk.get.tempfile.name" <- function (prefix="image", extension=".jpg") {
-	return (.rk.do.call ("get.tempfile.name", c (prefix, extension)))
+	return (.rk.do.plain.call ("get.tempfile.name", c (prefix, extension)))
+}
+
+"rk.get.workspace.url" <- function () {
+	res <- .rk.do.plain.call ("getWorkspaceUrl")
+	if (length (res)) res
+	else NULL
 }
 
 "rk.get.output.html.file" <- function () {
@@ -119,14 +132,47 @@
 	assign (".rk.output.html.file", x, as.environment ("package:rkward"))
 
 	if (!file.exists (x)) {
-		.rk.cat.output (paste ("<?xml version=\"1.0\" encoding=\"", .Call ("rk.locale.name"), "\"?>\n", sep=""));
-		.rk.cat.output ("<html><head><title>RKWard Output</title></head>\n<body>\n")
+		.rk.cat.output (paste ("<?xml version=\"1.0\" encoding=\"", .Call ("rk.locale.name"), "\"?>\n", sep=""))
+		.rk.cat.output (paste ("<html><head>\n<title>RKWard Output</title>\n", .rk.do.plain.call ("getCSSlink"), "</head>\n<body>\n", sep=""))
 		# This initial output mostly to indicate the output is really there, just empty for now
-		.rk.cat.output (paste ("<pre>RKWard output initialized on", date (), "</pre>\n"));
+		.rk.cat.output (paste ("<pre>RKWard output initialized on", date (), "</pre>\n"))
 	}
 
 	# needs to come after initialization, so initialization alone does not trigger an update during startup
-	.rk.do.call ("set.output.file", x);
+	.rk.do.plain.call ("set.output.file", x, synchronous=FALSE)
+	invisible (NULL)
+}
+
+"rk.save.workplace" <- function (file=NULL, description=NULL) {
+	if (is.null (file)) {
+		file <- rk.get.workspace.url ()
+		if (is.null (file)) file <- rk.get.tempfile.name (prefix="unsaved", extension=".RData")
+		file <- paste (file, "rkworkplace", sep=".")
+	}
+	if (is.null (description)) lines <- .rk.do.plain.call ("workplace.layout", "get")
+	else lines <- description
+	writeLines (lines, file)
+}
+
+"rk.restore.workplace" <- function (file=NULL, close.windows=TRUE) {
+	if (is.null (file)) {
+		if (exists (".rk.workplace.save", envir=globalenv (), inherits=FALSE)) {
+			# For backwards compatibility with workspaces saved by RKWard 0.5.4 and earlier.
+			# TODO: remove in time.
+			lines <- as.character (.GlobalEnv$.rk.workplace.save)
+			rm (list = c (".rk.workplace.save"), envir=globalenv ())
+		} else {
+			file <- rk.get.workspace.url ()
+			if (is.null (file)) file <- rk.get.tempfile.name (prefix="unsaved", extension=".RData")
+			file <- paste (file, "rkworkplace", sep=".")
+		}
+	}
+
+	close <- "close"
+	if (!isTRUE (close.windows)) close <- "noclose"
+	if (!exists ("lines", inherits=FALSE)) lines <- readLines (file)
+	.rk.do.plain.call ("workplace.layout", c ("set", close, lines), synchronous=FALSE)
+	invisible (NULL)
 }
 
 # renames a named object in a data.frame/list without changing it's position
@@ -151,6 +197,10 @@
 	}
 }
 
+"rk.print.code" <- function(code) {
+	.rk.cat.output (.rk.do.plain.call ("highlightRCode", as.character (code)))
+}
+
 "rk.header" <- function (title, parameters=list (), level=1) {
 	sink (rk.get.output.html.file(), append=TRUE)
 	on.exit (sink ())
@@ -337,7 +387,7 @@
 }
 
 "rk.show.html" <- function (url) {
-	invisible (.rk.do.call ("showHTML", as.character (url)));
+	invisible (.rk.do.plain.call ("showHTML", as.character (url), synchronous=FALSE));
 }
 
 "rk.call.plugin" <- function (plugin, ..., submit.mode = c ("manual", "auto", "submit")) {
@@ -374,7 +424,7 @@
 # list all available plugins in RKWard; this is a companion function for rk.call.plugin:
 # the output provides possible strings for "plugin" argument in rk.call.plugin
 rk.list.plugins <- function () {
-	.rk.do.call ("listPlugins")
+	.rk.do.plain.call ("listPlugins")
 }
 
 # a wrapper around chooseCRANmirror() without changing options ("repos"), permanently
@@ -420,7 +470,7 @@ rk.list.plugins <- function () {
 		}
 	}
 
-	.rk.do.call ("select.list", params)
+	.rk.do.plain.call ("select.list", params)
 }
 
 "rk.show.message" <- function (message, caption = "Information", wait=TRUE) {
@@ -436,3 +486,8 @@ rk.list.plugins <- function () {
 	else return (NULL)	# cancelled
 }
 
+"rk.sessionInfo" <- function () {
+	cat (.rk.do.plain.call ("getSessionInfo"), sep="\n")
+	cat ("R runtime session info:\n")
+	print (sessionInfo())
+}
