@@ -2,7 +2,7 @@
                           rkconsole  -  description
                              -------------------
     begin                : Thu Aug 19 2004
-    copyright            : (C) 2004, 2006, 2007, 2009, 2010 by Thomas Friedrichsmeier
+    copyright            : (C) 2004, 2006, 2007, 2009, 2010, 2011 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -40,6 +40,9 @@
 #include <ktexteditor/configinterface.h>
 #include <ktexteditor/markinterface.h>
 #include <kxmlguifactory.h>
+#include <kfiledialog.h>
+#include <kio/netaccess.h>
+#include <ktemporaryfile.h>
 
 #include "rkglobals.h"
 #include "rkward.h"
@@ -119,19 +122,18 @@ RKConsole::RKConsole (QWidget *parent, bool tool_window, const char *name) : RKM
 	nprefix = "> ";
 	iprefix = "+ ";
 	prefix = nprefix;
-	command_incomplete = false;
-	output_continuation = false;
 // KDE4: a way to do this?
 //	doc->setUndoSteps (0);
 	clear ();
-	doc->setHighlightingMode ("RKWard output");
+	RKCommandHighlighter::setHighlighting (doc, RKCommandHighlighter::RInteractiveSession);
 
 	commands_history = RKSettingsModuleConsole::loadCommandHistory ();
 	commands_history_position = commands_history.constEnd ();
 
 	current_command = 0;
+	current_command_displayed_up_to = 0;
 	tab_key_pressed_before = false;
-	command_was_piped = false;
+	previous_chunk_was_piped = false;
 
 // KDE4 TODO: workaround for KDE 4 pre-release versions. Hope this gets fixed before 4.0
 // see http://lists.kde.org/?l=kwrite-devel&m=119721420603507&w=2
@@ -199,10 +201,8 @@ void RKConsole::setCursorClear (int line, int col) {
 }
 
 bool RKConsole::handleKeyPress (QKeyEvent *e) {
-
 	KTextEditor::Cursor c = view->cursorPosition ();
 	int para=c.line (); int pos=c.column ();
-	command_was_piped = false;
 
 	if (para < doc->lines () - 1 || pos < prefix.length ()) {	// not inside the last line?
 		int key = e->key ();
@@ -231,6 +231,8 @@ bool RKConsole::handleKeyPress (QKeyEvent *e) {
 		return true;
 	}
 
+	previous_chunk_was_piped = false;
+
 	if (e->key () == Qt::Key_Up) {
 		commandsListUp (RKSettingsModuleConsole::shouldDoHistoryContextSensitive (e->modifiers ()));
 		return true;
@@ -279,6 +281,7 @@ bool RKConsole::handleKeyPress (QKeyEvent *e) {
 		return true;
 	} else if (e->key() == Qt::Key_Enter || e->key() == Qt::Key_Return) {
 		hinter->hideArgHint ();
+		addCommandToHistory (currentEditingLine ());
 		submitCommand ();
 		return true;
 	} else if (e->key () == Qt::Key_Left){
@@ -328,20 +331,18 @@ bool RKConsole::handleKeyPress (QKeyEvent *e) {
 	return false;
 }
 
-bool RKConsole::provideContext (unsigned int line_rev, QString *context, int *cursor_position) {
+QString RKConsole::provideContext (int line_rev) {
 	RK_TRACE (COMMANDEDITOR);
 
-	if (line_rev > 1) return false;
-
-	if (line_rev == 0) {
-		*cursor_position = currentCursorPositionInCommand ();
-		*context = currentCommand ();
-	} else {
-		*cursor_position = -1;
-		*context = incomplete_command;
+	QString ret;
+	if (line_rev == 0) ret = currentEditingLine ().left (currentCursorPositionInCommand ()); 
+	else if (!incomplete_command.isEmpty ()) {
+		QStringList lines = incomplete_command.split ("\n");
+		if (lines.size () > line_rev) {
+			ret = lines[lines.size () - line_rev - 1];
+		}
 	}
-
-	return true;
+	return ret;
 }
 
 void RKConsole::insertCompletion (int line_num, int word_start, int word_end, const QString &completion) {
@@ -416,59 +417,27 @@ bool RKConsole::doTabCompletionHelper (int line_num, const QString &line, int wo
 void RKConsole::doTabCompletion () {
 	RK_TRACE (APP);
 
-	QString current_line = currentCommand ();
+	QString current_line = currentEditingLine ();
 	int current_line_num = doc->lines () - 1;
 	int word_start;
 	int word_end;
 	int cursor_pos = currentCursorPositionInCommand ();
+	if (cursor_pos < 0) cursor_pos = current_line.length ();
 	RKCommonFunctions::getCurrentSymbolOffset (current_line, cursor_pos, false, &word_start, &word_end);
 	QString current_symbol = current_line.mid (word_start, word_end - word_start);
 
-	// should we try a file name completion? Let's do some heuristics
-	bool do_file_completion = false;
-	int quote_start = current_line.lastIndexOf ('"', cursor_pos - 1);
-	if (quote_start < 0) quote_start = current_line.lastIndexOf ('\'', cursor_pos - 1);
+	// as a very simple heuristic: If the current symbol starts with a quote, we should probably attempt file name completion, instead of symbol name completion
+	if (current_symbol.startsWith ("\"") || current_symbol.startsWith ("\'") || current_symbol.startsWith ("`")) {
+		KUrlCompletion comp (KUrlCompletion::FileCompletion);
+		comp.setDir (QDir::currentPath ());
+		comp.makeCompletion (current_symbol.mid (1));
 
-	if (quote_start >= 0) {
-		// we found a quoting char at some earlier position on the line, we might need a filename completion
-		do_file_completion = true;
+		if (doTabCompletionHelper (current_line_num, current_line, word_start + 1, word_end, comp.allMatches ())) return;
+	} else if (!current_symbol.isEmpty ()) {
+		RObject::RObjectSearchMap map;
+		RObjectList::getObjectList ()->findObjectsMatching (current_symbol, &map);
 
-		// however, some characters around quotes signify it's probably not really filename string
-		char char_before_quote = ' ';
-		if (quote_start > 1) char_before_quote = current_line.at (quote_start - 1).toLatin1();
-		char char_after_quote = ' ';
-		if (quote_start <= (current_line.length() - 2)) char_after_quote = current_line.at (quote_start + 1).toLatin1();
-		// these signifiy it might be an object in a list somewhere, e.g. my.data$"varname"
-		if ((char_before_quote == '[') || (char_before_quote == '$') || (char_before_quote == ':')) do_file_completion = false;
-		// these indicate, the quote has likely ended rather that started
-		if ((char_after_quote == ',') || (char_after_quote == ')') || (char_after_quote == ' ') || (char_after_quote == ';')) do_file_completion = false;
-
-		if (do_file_completion) {
-			int quote_end = current_line.indexOf ('"', cursor_pos);
-			if (quote_end < 0) quote_end = current_line.indexOf ('\'', cursor_pos);
-			if (quote_end < 0) quote_end = current_line.length ();
-	
-			QString current_name = current_line.mid (quote_start + 1, quote_end - quote_start - 1);
-			KUrlCompletion comp (KUrlCompletion::FileCompletion);
-			comp.setDir (QDir::currentPath ());
-			comp.makeCompletion (current_name);
-	
-			if (doTabCompletionHelper (current_line_num, current_line, quote_start+1, quote_end, comp.allMatches ())) return;
-		}
-	}
-
-	if (!do_file_completion) {
-		if (!current_symbol.isEmpty ()) {		// try object name completion first
-			RObject::RObjectSearchMap map;
-			RObject::RObjectSearchMap::const_iterator it;
-			RObjectList::getObjectList ()->findObjectsMatching (current_symbol, &map);
-	
-			QStringList entries;
-			for (it = map.constBegin (); it != map.constEnd (); ++it) {
-				entries.append (it.key ());
-			}
-			if (doTabCompletionHelper (current_line_num, current_line, word_start, word_end, entries)) return;
-		}
+		if (doTabCompletionHelper (current_line_num, current_line, word_start, word_end, map.keys ())) return;
 	}
 
 	// no completion was possible
@@ -514,12 +483,14 @@ bool RKConsole::eventFilter (QObject *o, QEvent *e) {
 	return false;
 }
 
-QString RKConsole::currentCommand () {
+QString RKConsole::currentEditingLine () const {
 	RK_TRACE (APP);
+
+	if (current_command) return QString ();	// it doesn't count as "editing" line, if a command is currently being executed
 	return (doc->line (doc->lines () - 1).mid (prefix.length ()));
 }
 
-void RKConsole::setCurrentCommand (const QString &command) {
+void RKConsole::setCurrentEditingLine (const QString &command) {
 	RK_TRACE (APP);
 
 	int lastline = doc->lines () - 1;
@@ -543,21 +514,37 @@ void RKConsole::cursorAtTheBeginning () {
 void RKConsole::submitCommand () {
 	RK_TRACE (APP);
 
-	QString current_line = currentCommand ();
-	QString command = current_line;
-	addCommandToHistory (current_line);
-	
-	if (command_incomplete) {
-		command.prepend (incomplete_command + '\n');
+	QString command = incomplete_command + currentEditingLine ();
+	if (!input_buffer.isEmpty ()) {
+		int last_line_end = input_buffer.lastIndexOf ('\n');
+		if (last_line_end < 0) {
+			last_line_end = 0;
+			RK_ASSERT (false);
+		}
+		command.append (input_buffer.left (last_line_end));
+		if (last_line_end < (input_buffer.size () - 1)) {
+			input_buffer = input_buffer.mid (last_line_end + 1);
+		} else {
+			input_buffer.clear ();
+		}
+	} else {
+		RK_ASSERT (!command.endsWith ('\n'));
+		command.append ('\n');
 	}
 
+	current_command_displayed_up_to = incomplete_command.length ();
+	setCurrentEditingLine (command.mid (current_command_displayed_up_to, command.indexOf ('\n', current_command_displayed_up_to) - current_command_displayed_up_to));
+	current_command_displayed_up_to += currentEditingLine ().length ();
+	skip_command_display_lines = incomplete_command.count ('\n') + 1;	// incomplete command, and first line have already been shown.
+
 	doc->insertLine (doc->lines (), QString ());
-	if (!current_line.isEmpty ()) {
-		current_command = new RCommand (command, RCommand::User | RCommand::Console, QString::null, this);
+	if (!command.isEmpty ()) {
+		current_command = new RCommand (command, RCommand::User | RCommand::Console, QString (), this);
 		RKGlobals::rInterface ()->issueCommand (current_command);
 		interrupt_command_action->setEnabled (true);
 	} else {
-		tryNextInBatch ();
+		showPrompt ();
+		tryNextInBuffer ();
 	}
 }
 
@@ -565,11 +552,11 @@ void RKConsole::commandsListUp (bool context_sensitive) {
 	RK_TRACE (APP);
 
 	// if we are at the last line, i.e. not yet navigating the command history, store the current command
-	if (commands_history.constEnd () == commands_history_position) history_editing_line = currentCommand ();
+	if (commands_history.constEnd () == commands_history_position) history_editing_line = currentEditingLine ();
 
 	if (context_sensitive) {
 		if (command_edited) {
-			command_history_context = currentCommand ();
+			command_history_context = currentEditingLine ();
 			commands_history_position = commands_history.constEnd ();
 			command_edited = false;
 		}
@@ -589,7 +576,7 @@ void RKConsole::commandsListUp (bool context_sensitive) {
 
 	if (found) {		// if we did not find a previous matching line, do not touch the commands_history_position
 		commands_history_position = it;
-		setCurrentCommand (*commands_history_position);
+		setCurrentEditingLine (*commands_history_position);
 	} else {
 		KApplication::kApplication ()->beep ();
 	}
@@ -600,7 +587,7 @@ void RKConsole::commandsListDown (bool context_sensitive) {
 
 	if (context_sensitive) {
 		if (command_edited) {
-			command_history_context = currentCommand ();
+			command_history_context = currentEditingLine ();
 	  		commands_history_position = commands_history.constEnd ();
 	  		command_edited = false;
 	  		return; // back at bottommost item
@@ -620,41 +607,38 @@ void RKConsole::commandsListDown (bool context_sensitive) {
 		}
 	}
 
-	if (commands_history.constEnd () == commands_history_position) setCurrentCommand (history_editing_line);
-	else setCurrentCommand (*commands_history_position);
+	if (commands_history.constEnd () == commands_history_position) setCurrentEditingLine (history_editing_line);
+	else setCurrentEditingLine (*commands_history_position);
 }
 
 void RKConsole::rCommandDone (RCommand *command) {
 	RK_TRACE (APP);
+
+	current_command = 0;
+
 	if (command->errorSyntax () && command->error ().isEmpty ()) {
 		doc->insertLine (doc->lines () - 1, i18n ("Syntax error\n"));
 	}
 
 	if (command->errorIncomplete ()) {
 		prefix = iprefix;
-		command_incomplete = true;
-		incomplete_command = command->command ();
+		incomplete_command = command->remainingCommand ();
 	} else {
 		prefix = nprefix;
-		command_incomplete = false;
-		incomplete_command = QString::null;
+		incomplete_command.clear ();
 	}
 
-	if (output_continuation) doc->insertLine (doc->lines (), "");
-	output_continuation = false;
 	commands_history_position = commands_history.constEnd ();
-	tryNextInBatch ();
+	showPrompt ();
+	tryNextInBuffer ();
 }
 
 void RKConsole::newOutput (RCommand *, ROutput *output) {
 	RK_TRACE (APP);
 
 	int start_line = doc->lines () -1;
-	if (output_continuation) {
-		doc->insertText (doc->documentEnd (), output->output);
-	} else {
-		doc->insertText (KTextEditor::Cursor (doc->lines () -1, 0), output->output);
-	}
+	doc->insertText (doc->documentEnd (), output->output);
+
 	int end_line = doc->lines () -1;
 	if (output->type != ROutput::Output) {
 		KTextEditor::MarkInterface *markiface = qobject_cast<KTextEditor::MarkInterface*> (doc);
@@ -666,7 +650,7 @@ void RKConsole::newOutput (RCommand *, ROutput *output) {
 
 	if (RKSettingsModuleConsole::maxConsoleLines ()) {
 		uint c = (uint) doc->lines();
-// We remove the superflous lines in chunks of 20 while handling output for better performance. Later, in tryNextInBatch(), we trim down to the correct size.
+// We remove the superflous lines in chunks of 20 while handling output for better performance. Later, in showPrompt(), we trim down to the correct size.
 		if (c > (RKSettingsModuleConsole::maxConsoleLines () + 20)) {
 // KDE4: does the setUpdatesEnabled (false) still affect performance?
 			view->setUpdatesEnabled (false);		// major performance boost while removing lines!
@@ -675,22 +659,44 @@ void RKConsole::newOutput (RCommand *, ROutput *output) {
 		}
 	}
 	cursorAtTheEnd ();
-	output_continuation = true;
+}
+
+void RKConsole::userCommandLineIn (RCommand* cmd) {
+	RK_TRACE (APP);
+	RK_ASSERT (cmd == current_command);
+
+	if (--skip_command_display_lines >= 0) return;
+
+	QString line = cmd->command ().mid (current_command_displayed_up_to + 1);
+	line = line.section ('\n', 0, 0) + '\n';
+	current_command_displayed_up_to += line.length ();
+	if (line.length () < 2) return;		// omit empty lines (esp. the trailing newline of the command!)
+
+	prefix = iprefix;
+	showPrompt ();
+	setCurrentEditingLine (line);
 }
 
 void RKConsole::submitBatch (const QString &batch) {
 	RK_TRACE (APP);
 
-	if (current_command) return;
-	// splitting batch, not allowing empty entries.
-	// TODO: hack something so we can have empty entries.
-	commands_batch = batch.split ("\n", QString::SkipEmptyParts);
-	tryNextInBatch (false);
+	previous_chunk_was_piped = false;
+	input_buffer.append (batch);
+	if (!current_command) {		// pasting into the middle of the command line
+		QString line = currentEditingLine ();
+		int pos = currentCursorPositionInCommand ();
+		if (pos >= 0) {
+			setCurrentEditingLine (line.left (pos));
+			input_buffer.append (line.mid (pos));
+		}
+	}
+	if (!current_command) tryNextInBuffer ();
 }
 
-void RKConsole::tryNextInBatch (bool add_new_line) {
+void RKConsole::showPrompt () {
 	RK_TRACE (APP);
-	if (add_new_line) {
+
+	if ((!doc->lines ()) || (!doc->line (doc->lines () - 1).isEmpty ())) {
 		if (RKSettingsModuleConsole::maxConsoleLines ()) {
 			uint c = doc->lines();
 			if (c > RKSettingsModuleConsole::maxConsoleLines ()) {
@@ -700,27 +706,34 @@ void RKConsole::tryNextInBatch (bool add_new_line) {
 				view->setUpdatesEnabled (true);
 			}
 		}
-		if (!doc->lines ()) doc->insertLine (0, QString ());
-		doc->insertText (KTextEditor::Cursor (doc->lines () - 1, 0), prefix);
-//		doc->insertLine (doc->lines (), prefix);		// somehow, it seems to be safer to do this after removing superfluous lines, than before
-		cursorAtTheEnd ();
+		doc->insertLine (doc->lines (), QString ());
 	}
+	doc->insertText (KTextEditor::Cursor (doc->lines () - 1, 0), prefix);
+//	doc->insertLine (doc->lines (), prefix);		// somehow, it seems to be safer to do this after removing superfluous lines, than before
+	cursorAtTheEnd ();
+}
 
-	if (!commands_batch.isEmpty()) {
-		// If we were not finished executing a batch of commands, we execute the next one.
-		setCurrentCommand (currentCommand () + commands_batch.first ());
-		commands_batch.pop_front ();
-		if (!commands_batch.isEmpty ()){
-			submitCommand ();
+void RKConsole::tryNextInBuffer () {
+	RK_TRACE (APP);
+
+	if (!input_buffer.isEmpty ()) {
+		if (input_buffer.contains ('\n')) {
+			submitCommand ();	// will submit and clear the buffer
 			return;
+		} else {
+			setCurrentEditingLine (currentEditingLine () + input_buffer);
+			input_buffer.clear ();
 		}
-		// We would put this here if we would want the last line to be executed. We generally don't want this, as there is an empty last item, if there is a newline at the end.
-		//TODO: deal with this kind of situation better.
-		//commands_batch.erase(commands_batch.begin());
 	}
+	interrupt_command_action->setEnabled (!incomplete_command.isEmpty ());
+}
 
-	current_command = 0;
-	interrupt_command_action->setEnabled (isBusy ());
+bool RKConsole::isBusy () const {
+	if (current_command) return true;
+	if (!incomplete_command.isEmpty ()) return true;
+	if (!currentEditingLine ().isEmpty ()) return true;
+	if (!input_buffer.isEmpty ()) return true;
+	return false;
 }
 
 void RKConsole::paste () {
@@ -732,15 +745,14 @@ void RKConsole::paste () {
 void RKConsole::clear () {
 	RK_TRACE (APP);
 	doc->clear ();
-	tryNextInBatch ();
+	showPrompt ();
 }
 
 void RKConsole::addCommandToHistory (const QString &command) {
 	RK_TRACE (APP);
-	if (command.isEmpty () || ((!commands_history.isEmpty ()) && commands_history.last() == command)) return; // don't add empty or duplicate lines
-
-	commands_history.append (command);
-	history_editing_line = QString ();
+	if ((!command.isEmpty ()) && (commands_history.isEmpty () || commands_history.last() != command)) { // don't add empty or duplicate lines
+		commands_history.append (command);
+	}
 
 	if (RKSettingsModuleConsole::maxHistoryLength ()) {
 		uint c = commands_history.count ();
@@ -748,25 +760,100 @@ void RKConsole::addCommandToHistory (const QString &command) {
 			commands_history.pop_front ();
 		}
 	}
+
+	history_editing_line = QString ();
+	commands_history_position = commands_history.constEnd ();
+	command_edited = false;
 }
 
-QString RKConsole::cleanedSelection () {
+void RKConsole::setCommandHistory (const QStringList &new_history, bool append) {
 	RK_TRACE (APP);
 
-	QString ret = view->selectionText ();
-	ret.replace ('\n' + QString (nprefix), "\n");
-	ret.replace ('\n' + QString (iprefix), "\n");
-	if (ret.startsWith (nprefix)) {
-		ret = ret.mid (QString (nprefix).length ());
+	if (append) commands_history.append (new_history);
+	else commands_history = new_history;
+
+	addCommandToHistory (QString ());	// side-effect of checking history length
+}
+
+void RKConsole::userLoadHistory (const KUrl &_url) {
+	RK_TRACE (APP);
+
+	KUrl url = _url;
+	if (url.isEmpty ()) {
+		url = KFileDialog::getOpenUrl (KUrl (), i18n ("*.Rhistory|R history files (*.Rhistory)\n*|All files (*)"), this, i18n ("Select command history file to load"));
+		if (url.isEmpty ()) return;
+	}
+
+	QString tempfile;
+	KIO::NetAccess::download (url, tempfile, this);
+
+	QFile file (tempfile);
+	if (!file.open (QIODevice::Text | QIODevice::ReadOnly)) return;
+	setCommandHistory (QString (file.readAll ()).split ('\n', QString::SkipEmptyParts), false);
+	file.close ();
+
+	KIO::NetAccess::removeTempFile (tempfile);
+}
+
+void RKConsole::userSaveHistory (const KUrl &_url) {
+	RK_TRACE (APP);
+
+	KUrl url = _url;
+	if (url.isEmpty ()) {
+		url = KFileDialog::getSaveUrl (KUrl (), i18n ("*.Rhistory|R history files (*.Rhistory)\n*|All files (*)"), this, i18n ("Select filename to save command history")
+#if KDE_IS_VERSION(4,4,0)
+		                                    , KFileDialog::ConfirmOverwrite
+#endif
+		                                   );
+		if (url.isEmpty ()) return;
+	}
+
+	KTemporaryFile tempfile;
+	tempfile.open ();
+	tempfile.write (QString (commandHistory ().join ("\n") + "\n").toLocal8Bit ().data ());
+	tempfile.close ();
+
+	KIO::NetAccess::upload (tempfile.fileName (), url, this);
+}
+
+QString RKConsole::cleanSelection (const QString &origin) {
+	RK_TRACE (APP);
+
+	QString ret;
+	ret.reserve (origin.length ());
+	QStringList lines = origin.split ("\n");
+	foreach (QString line, lines) {
+		if (line.startsWith (nprefix)) {
+			ret.append (line.mid (nprefix.length ()));
+		} else if (line.startsWith (iprefix)) {
+			ret.append (line.mid (iprefix.length ()));
+		} else {
+			ret.append (line);
+		}
+		ret.append ('\n');
 	}
 
 	return ret;
 }
 
-void RKConsole::copy () {
+void RKConsole::copyCommands () {
 	RK_TRACE (APP);
 
-	QApplication::clipboard()->setText (cleanedSelection ());
+	KTextEditor::Range sel = view->selectionRange ();
+	if (!sel.isValid ()) return;
+
+	// we use this somewhat cumbersome (and inefficient) approach as it should also be correct in case of blockwise selections
+	QStringList command_lines = view->selectionText ().split ("\n");
+	int i = 0;
+	for (int l = sel.start ().line (); l <= sel.end ().line (); ++l) {
+		QString line = doc->line (l);
+		if (!(line.startsWith (iprefix) || line.startsWith (nprefix))) {
+			command_lines.removeAt (i);
+			--i;
+		}
+		++i;
+	}
+	QApplication::clipboard()->setText (cleanSelection (command_lines.join ("\n")));
 }
 
 void RKConsole::literalCopy () {
@@ -775,72 +862,66 @@ void RKConsole::literalCopy () {
 	QApplication::clipboard()->setText (view->selectionText ());
 }
 
-int RKConsole::currentCursorPosition (){
-	KTextEditor::Cursor c = view->cursorPosition ();
-	return(c.column ());
-}
-
 int RKConsole::currentCursorPositionInCommand(){
 	RK_TRACE (APP);
-	return(currentCursorPosition() - prefix.length());
+
+	KTextEditor::Cursor c = view->cursorPosition ();
+	if (c.line () < (doc->lines () - 1)) return -1;
+	return (c.column () - prefix.length());
 }
 
-void RKConsole::resetIncompleteCommand () {
+void RKConsole::resetConsole () {
 	RK_TRACE (APP);
-
-	RK_ASSERT (command_incomplete);
-	prefix = nprefix;
-	command_incomplete = false;
-	incomplete_command = QString::null;
-	doc->insertLine (doc->lines (), "");
-
-	tryNextInBatch (true);
-}
-
-void RKConsole::slotInterruptCommand () {
-	RK_TRACE (APP);
-	RK_ASSERT (current_command || command_incomplete);
 	RK_DO (qDebug("received interrupt signal in console"), APP, DL_DEBUG);
 
-	commands_batch.clear ();
-	if (command_incomplete) {
-		resetIncompleteCommand ();
-	} else {
+	input_buffer.clear ();
+	if (current_command) {
 		RKGlobals::rInterface ()->cancelCommand (current_command);
+	} else {
+		prefix = nprefix;
+		incomplete_command.clear ();
+
+		showPrompt ();
+		tryNextInBuffer ();		// well, the buffer is empty, but this also sets the interrupt_command_action's state, etc.
 	}
 }
 
 void RKConsole::runSelection () {
 	RK_TRACE (APP);
 
-	QString command = cleanedSelection ();
-	pipeUserCommand (new RCommand (command, RCommand::User));
+	pipeUserCommand (cleanSelection (view->selectionText ()));
+}
+
+void RKConsole::copyLinesToOutput () {
+	RK_TRACE (APP);
+
+	RKCommandHighlighter::copyLinesToOutput (view, RKCommandHighlighter::RInteractiveSession);
 }
 
 void RKConsole::showContextHelp () {
 	RK_TRACE (APP);
-	RKHelpSearchWindow::mainHelpSearch ()->getContextHelp (currentCommand (), currentCursorPositionInCommand ());
+	RKHelpSearchWindow::mainHelpSearch ()->getContextHelp (currentEditingLine (), currentCursorPositionInCommand ());
 }
 
 void RKConsole::initializeActions (KActionCollection *ac) {
 	RK_TRACE (APP);
 
+	RKStandardActions::copyLinesToOutput (this, this, SLOT (copyLinesToOutput()));
 	context_help_action = RKStandardActions::functionHelp (this, this, SLOT(showContextHelp()));
-
 	run_selection_action = RKStandardActions::runSelection (this, this, SLOT (runSelection()));
 
-	interrupt_command_action = ac->addAction ("interrupt", this, SLOT (slotInterruptCommand()));
+	interrupt_command_action = ac->addAction ("interrupt", this, SLOT (resetConsole()));
 	interrupt_command_action->setText (i18n ("Interrupt running command"));
 	interrupt_command_action->setShortcut (Qt::ControlModifier + Qt::Key_C);
-	interrupt_command_action->setIcon (KIcon ("media-playback-stop"));
+	interrupt_command_action->setIcon (RKStandardIcons::getIcon (RKStandardIcons::ActionInterrupt));
 	interrupt_command_action->setEnabled (false);
 
-	copy_action = ac->addAction ("rkconsole_copy", this, SLOT (copy()));
-	copy_action->setText (i18n ("Copy selection cleaned"));
-
 	copy_literal_action = ac->addAction ("rkconsole_copy_literal", this, SLOT (literalCopy()));
 	copy_literal_action->setText (i18n ("Copy selection literally"));
 
+	copy_commands_action = ac->addAction ("rkconsole_copy_commands", this, SLOT (copyCommands()));
+	copy_commands_action->setText (i18n ("Copy commands, only"));
+
 	RKStandardActions::pasteSpecial (this, this, SLOT (submitBatch(const QString&)));
 
 	ac->addAction (KStandardAction::Clear, "rkconsole_clear", this, SLOT (clear()));
@@ -851,66 +932,53 @@ void RKConsole::initializeActions (KActionCollection *ac) {
 	addProxyAction ("view_dynamic_word_wrap");
 	addProxyAction ("view_inc_font_sizes");
 	addProxyAction ("view_dec_font_sizes");
+
+	KAction *action = ac->addAction ("loadhistory", this, SLOT (userLoadHistory ()));
+	action->setText (i18n ("Import command history..."));
+	action = ac->addAction ("savehistory", this, SLOT (userSaveHistory ()));
+	action->setText (i18n ("Export command history..."));
 }
 
 void RKConsole::pipeUserCommand (const QString &command) {
 	RK_TRACE (APP);
 
-	RCommand *cmd = new RCommand (command, RCommand::User);
-	pipeUserCommand (cmd);
-}
-
-void RKConsole::pipeUserCommand (RCommand *command) {
-	RK_TRACE (APP);
-
 	if (RKSettingsModuleConsole::pipeUserCommandsThroughConsole ()) {
 		RKConsole::mainConsole ()->pipeCommandThroughConsoleLocal (command);
 	} else {
-		RKGlobals::rInterface ()->issueCommand (command);
+		RCommand *cmd = new RCommand (command, RCommand::User);
+		RKGlobals::rInterface ()->issueCommand (cmd);
 	}
 }
 
-void RKConsole::pipeCommandThroughConsoleLocal (RCommand *command) {
+void RKConsole::pipeCommandThroughConsoleLocal (const QString &command_string) {
 	RK_TRACE (APP);
 
 	activate (false);
-	if ((!command_was_piped) && (isBusy () || (!currentCommand ().isEmpty ()))) {
-		int res = KMessageBox::questionYesNo (this, i18n ("You have configured RKWard to run script commands through the console. However, the console is currently busy (either a command is running, or you have started to enter text in the console). Do you want to bypass the console this one time, or do you want to try again later?"), i18n ("Console is busy"), KGuiItem (i18n ("Bypass console")), KGuiItem (i18n ("Cancel")));
-		if (res == KMessageBox::Yes) {
-			RKGlobals::rInterface ()->issueCommand (command);
-		} else {
-			delete command;
+	if (isBusy () && (!previous_chunk_was_piped)) {
+		int res = KMessageBox::questionYesNoCancel (this, i18n ("You have configured RKWard to pipe script editor commands through the R Console. However, another command is currently active in the console. Do you want to append it to the command in the console, or do you want to reset the console, first? Press cancel if you do not wish to run the new command, now."), i18n ("R Console is busy"), KGuiItem (i18n ("Append")), KGuiItem (i18n ("Reset, then submit")));
+		if (res == KMessageBox::No) {
+			resetConsole ();
+		} else if (res != KMessageBox::Yes) {
+			return;
 		}
-	} else {
-		QString command_string = command->command ();
-		QString text = command_string;
-		if (RKSettingsModuleConsole::addPipedCommandsToHistory() != RKSettingsModuleConsole::DontAdd) {
-			QStringList lines = text.split ('\n', QString::SkipEmptyParts);
-			if ((RKSettingsModuleConsole::addPipedCommandsToHistory() == RKSettingsModuleConsole::AlwaysAdd) || (lines.count () == 1)) {
-				for (int i = 0; i < lines.count (); ++i) {
-					addCommandToHistory (lines[i]);
-				}
+	}
+	if (RKSettingsModuleConsole::addPipedCommandsToHistory() != RKSettingsModuleConsole::DontAdd) {
+		QStringList lines = command_string.split ('\n', QString::SkipEmptyParts);
+		if ((RKSettingsModuleConsole::addPipedCommandsToHistory() == RKSettingsModuleConsole::AlwaysAdd) || (lines.count () == 1)) {
+			for (int i = 0; i < lines.count (); ++i) {
+				addCommandToHistory (lines[i]);
 			}
 		}
-		text.replace ('\n', QString ("\n") + iprefix);
-		doc->insertText (KTextEditor::Cursor (doc->lines () - 1, QString (nprefix).length ()), text + '\n');
-		command->addReceiver (this);
-		command->addTypeFlag (RCommand::Console);
-		current_command = command;
-		if (command_incomplete) {
-			RK_ASSERT (command_was_piped);
-			command_string.prepend (incomplete_command + '\n');
-			command->setCommand (command_string);
-		}
-		command_was_piped = true;
-		RKGlobals::rInterface ()->issueCommand (command);
 	}
+	cursorAtTheEnd ();
+	submitBatch (command_string + '\n');
+	previous_chunk_was_piped = true;
 }
 
 void RKConsole::contextMenuEvent (QContextMenuEvent * event) {
 	RK_TRACE (APP);
 
-	copy_action->setEnabled (view->selection ());
+	copy_commands_action->setEnabled (view->selection ());
 	copy_literal_action->setEnabled (view->selection ());
 	run_selection_action->setEnabled (view->selection ());
 
@@ -918,7 +986,7 @@ void RKConsole::contextMenuEvent (QContextMenuEvent * event) {
 
 	run_selection_action->setEnabled (true);
 	copy_literal_action->setEnabled (true);
-	copy_action->setEnabled (true);
+	copy_commands_action->setEnabled (true);
 }
 
 void RKConsole::activate (bool with_focus) {
