@@ -2,7 +2,7 @@
                           rkward.cpp  -  description
                              -------------------
     begin                : Tue Oct 29 20:06:08 CET 2002
-    copyright            : (C) 2002, 2005, 2006, 2007, 2008, 2009, 2010 by Thomas Friedrichsmeier 
+    copyright            : (C) 2002, 2005, 2006, 2007, 2008, 2009, 2011 by Thomas Friedrichsmeier 
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -47,6 +47,7 @@
 #include <krecentfilesaction.h>
 #include <khbox.h>
 #include <ktoolbar.h>
+#include <kactionmenu.h>
 
 // application specific includes
 #include "rkward.h"
@@ -66,6 +67,7 @@
 #include "dialogs/startupdialog.h"
 #include "dialogs/rkloadlibsdialog.h"
 #include "dialogs/rkimportdialog.h"
+#include "dialogs/rkrecoverdialog.h"
 #include "agents/rksaveagent.h"
 #include "agents/rkloadagent.h"
 #include "agents/rkquitagent.h"
@@ -78,6 +80,7 @@
 #include "windows/rkhelpsearchwindow.h"
 #include "windows/rktoplevelwindowgui.h"
 #include "windows/rkfilebrowser.h"
+#include "windows/rktoolwindowlist.h"
 #include "rkconsole.h"
 #include "debug.h"
 #include "version.h"
@@ -110,6 +113,7 @@ RKWardMainWindow::RKWardMainWindow (RKWardStartupOptions *options) : KParts::Mai
 	RK_TRACE (APP);
 	RK_ASSERT (rkward_mainwin == 0);
 
+	gui_rebuild_locked = true;
 	rkward_mainwin = this;
 	RKGlobals::rinter = 0;
 	RKSettings::settings_tracker = new RKSettingsTracker (this);
@@ -121,10 +125,19 @@ RKWardMainWindow::RKWardMainWindow (RKWardStartupOptions *options) : KParts::Mai
 	initStatusBar();
 
 	new RKWorkplace (this);
-	RKWorkplace::mainWorkplace ()->initActions (actionCollection (), "prev_window", "next_window", "left_window", "right_window");
+	RKWorkplace::mainWorkplace ()->initActions (actionCollection (), "left_window", "right_window");
 	setCentralWidget (RKWorkplace::mainWorkplace ());
 	connect (RKWorkplace::mainWorkplace ()->view (), SIGNAL (captionChanged (const QString &)), this, SLOT (setCaption (const QString &)));
 
+	part_manager = new KParts::PartManager (this);
+	// When the manager says the active part changes,
+	// the builder updates (recreates) the GUI
+	connect (partManager (), SIGNAL (activePartChanged (KParts::Part *)), this, SLOT (partChanged (KParts::Part *)));
+
+	readOptions();
+	RKGlobals::mtracker = new RKModificationTracker (this);
+	initToolViewsAndR ();
+
 	///////////////////////////////////////////////////////////////////
 	// build the interface
 
@@ -134,37 +147,29 @@ RKWardMainWindow::RKWardMainWindow (RKWardStartupOptions *options) : KParts::Mai
 	createShellGUI (true);
 	RKXMLGUISyncer::self ()->watchXMLGUIClientUIrc (this);
 
-	RKGlobals::mtracker = new RKModificationTracker (this);
+	proxy_import->setMenu (dynamic_cast<QMenu*>(guiFactory ()->container ("import", this)));
+	proxy_export->setMenu (dynamic_cast<QMenu*>(guiFactory ()->container ("export", this)));
+
 	RKComponentMap::initialize ();
 
-	if (options) {
-		startup_options = options;
-	} else {
-		startup_options = new RKWardStartupOptions;
-		startup_options->no_stack_check = false;
-		startup_options->initial_url = KUrl();
-	}
+	startup_options = options;
 
-	QTimer::singleShot (50, this, SLOT (doPostInit ()));
-
-	part_manager = new KParts::PartManager (this);
-	// When the manager says the active part changes,
-	// the builder updates (recreates) the GUI
-	connect (partManager (), SIGNAL (activePartChanged (KParts::Part *)), this, SLOT (partChanged (KParts::Part *)));
+	// stuff which should wait until the event loop is running
+	QTimer::singleShot (0, this, SLOT (doPostInit ()));
 }
 
 RKWardMainWindow::~RKWardMainWindow() {
 	RK_TRACE (APP);
 
 	// these would not be strictly necessary, as we're exiting the app, anyway.
-	delete RControlWindow::getControl ();
-	delete RKGlobals::rInterface ();
 	delete RObjectList::getObjectList ();
 	delete RObjectBrowser::mainBrowser ();
 	delete RKCommandLog::getLog ();
 	delete RKConsole::mainConsole ();
 	delete RKHelpSearchWindow::mainHelpSearch ();
 	delete RKGlobals::tracker ();
+	delete RKGlobals::rInterface ();
+	delete RControlWindow::getControl ();
 }
 
 void RKWardMainWindow::closeEvent (QCloseEvent *e) {
@@ -187,70 +192,53 @@ void RKWardMainWindow::doPostInit () {
 
 	// Check installation first
 	QFile resource_ver (RKCommonFunctions::getRKWardDataDir () + "resource.ver");
-	if (!(resource_ver.open (QIODevice::ReadOnly) && (resource_ver.read (100).trimmed () == VERSION))) {
+	if (!(resource_ver.open (QIODevice::ReadOnly) && (resource_ver.read (100).trimmed () == RKWARD_VERSION))) {
 		KMessageBox::error (this, i18n ("<p>RKWard either could not find its resource files at all, or only an old version of those files. The most likely cause is that the last installation failed to place the files in the correct place. This can lead to all sorts of problems, from single missing features to complete failure to function.</p><p><b>You should quit RKWard, now, and fix your installation</b>. For help with that, see <a href=\"http://p.sf.net/rkward/compiling\">http://p.sf.net/rkward/compiling</a>.</p>"), i18n ("Broken installation"), KMessageBox::Notify | KMessageBox::AllowLink);
 	}
 
+	// startup options will be deleted from the R thread (TODO correct this!), so we need to copy the initial_url here, or run into race conditions
+	KUrl open_url = startup_options ? startup_options->initial_url : KUrl ();
+	QString evaluate_code = startup_options ? startup_options->evaluate : QString ();
+	delete startup_options;
+	startup_options = 0;
+
+	initPlugins ();
+	gui_rebuild_locked = false;
+
+	show ();
 #ifdef Q_WS_WIN
-	KMessageBox::information (this, i18n ("<p>Please note that RKWard is not well tested on Windows, yet.</p><p>There are all sorts of known issues, and issues yet to be reported. If you'd like to help: <a href=\"http://p.sf.net/rkward/contact\">http://p.sf.net/rkward/contact</a>.</p>"), i18n ("RKWard on Windows"), "rkward_on_windows", KMessageBox::Notify | KMessageBox::AllowLink);
+	// detect and disable the buggy "native" file dialogs
+	KConfigGroup cg = KGlobal::config ().data ()->group ("KFileDialog Settings");
+	if (cg.readEntry ("Native", true)) {
+		int res = KMessageBox::questionYesNo (this, i18n ("Your installation of KDE is configured to use \"native\" file dialogs. This is known to cause issues in some cases, and we recommend to disable \"native\" file dialogs.\nShould \"native\" file dialogs be disabled in RKWard?"),
+							i18n ("Potential problem with your configuration"), KGuiItem (i18n ("Yes, disable")), KGuiItem (i18n ("No, use \"native\" file dialogs")), "windows_native_kfiledialog");
+		if (res != KMessageBox::No) {
+			cg.writeEntry ("Native", false);
+			cg.sync ();
+		}
+	}
 #endif
 
-	setUpdatesEnabled (false);
-
-	readOptions();
-	//It's necessary to give a different name to all tool windows, or they won't be properly displayed
-	RObjectBrowser::object_browser = new RObjectBrowser (0, true, "workspace");
-
-	RKCommandLog *log = new RKCommandLog (0, true, "Command log");
-	RKWorkplace::mainWorkplace ()->placeInToolWindowBar (log, KMultiTabBar::Bottom);
-	RKCommandLog::rkcommand_log = log;
-
-	// startup options will be deleted from the R thread (TODO correct this!), so we need to copy the initial_url here, or run into race conditions
-	KUrl open_url = startup_options->initial_url;
-	QString evaluate_code = startup_options->evaluate;
-	startR ();
-	
-	initPlugins ();
-
-	RObjectBrowser::mainBrowser ()->setCaption (i18n ("Workspace"));
-	RKWorkplace::mainWorkplace ()->placeInToolWindowBar (RObjectBrowser::mainBrowser (), KMultiTabBar::Left);
-
-	RKFileBrowser::main_browser = new RKFileBrowser (0, true, "file_browser");
-	RKFileBrowser::main_browser->setCaption (i18n ("Files"));
-	RKWorkplace::mainWorkplace ()->placeInToolWindowBar (RKFileBrowser::main_browser, KMultiTabBar::Left);
-
-	RControlWindow::control_window = new RControlWindow (0, true, "rcontrol");
-	RControlWindow::getControl ()->setCaption (i18n ("Pending Jobs"));
-	RKWorkplace::mainWorkplace ()->placeInToolWindowBar (RControlWindow::getControl (), KMultiTabBar::Bottom);
-
-	RKConsole *console = new RKConsole (0, true, "r_console");
-	RKConsole::setMainConsole (console);
-	RKWorkplace::mainWorkplace ()->placeInToolWindowBar (console, KMultiTabBar::Bottom);
-
-	RKHelpSearchWindow *help_search = new RKHelpSearchWindow (0, true, "r_help");
-	RKHelpSearchWindow::main_help_search = help_search;
-	RKWorkplace::mainWorkplace ()->placeInToolWindowBar (help_search, KMultiTabBar::Bottom);
-
-	setUpdatesEnabled (true);
-	show ();
+	KUrl recover_url = RKRecoverDialog::checkRecoverCrashedWorkspace ();
+	if (!recover_url.isEmpty ()) open_url = recover_url;
 
 	if (!open_url.isEmpty()) {
 		openWorkspace (open_url);
 	} else {
-		StartupDialog::StartupDialogResult *result = StartupDialog::getStartupAction (this, fileOpenRecentWorkspace);
-		if (result->result == StartupDialog::EmptyWorkspace) {
-		} else if (result->result == StartupDialog::OpenFile) {
-			openWorkspace (result->open_url);
-		} else if (result->result == StartupDialog::ChoseFile) {
-			slotFileOpenWorkspace ();
-		} else if (result->result == StartupDialog::EmptyTable) {
-			RKWorkplace::mainWorkplace ()->editNewDataFrame (i18n ("my.data"));
+		StartupDialog::StartupDialogResult result = StartupDialog::getStartupAction (this, fileOpenRecentWorkspace);
+		if (!result.open_url.isEmpty ()) {
+			openWorkspace (result.open_url);
+		} else {
+			if (result.result == StartupDialog::ChoseFile) {
+				slotFileOpenWorkspace ();
+			} else if (result.result == StartupDialog::EmptyTable) {
+				RKWorkplace::mainWorkplace ()->editNewDataFrame (i18n ("my.data"));
+			}
 		}
-		delete result;
 	}
 
 	if (RKSettingsModuleGeneral::workplaceSaveMode () == RKSettingsModuleGeneral::SaveWorkplaceWithSession) {
-		RKWorkplace::mainWorkplace ()->restoreWorkplace (RKSettingsModuleGeneral::getSavedWorkplace (KGlobal::config ().data ()));
+		RKWorkplace::mainWorkplace ()->restoreWorkplace (RKSettingsModuleGeneral::getSavedWorkplace (KGlobal::config ().data ()).split ('\n'));
 	}
 
 	if (RKSettingsModuleGeneral::showHelpOnStartup ()) {
@@ -296,17 +284,66 @@ void RKWardMainWindow::startR () {
 	new RObjectList ();
 	connect (RObjectList::getObjectList (), SIGNAL (workspaceUrlChanged(const KUrl&)), this, SLOT (addWorkspaceUrl(const KUrl&)));
 
-	RKGlobals::rInterface ()->startThread ();
-
 	RObjectBrowser::mainBrowser ()->unlock ();
 }
 
 void RKWardMainWindow::slotConfigure () {
+	RK_TRACE (APP);
 	RKSettings::configureSettings (RKSettings::NoPage, this);
 }
 
-void RKWardMainWindow::initActions()
-{  
+void RKWardMainWindow::slotCancelAllCommands () {
+	RK_TRACE (APP);
+	RK_ASSERT (RKGlobals::rInterface ());
+	RKGlobals::rInterface ()->cancelAll ();
+}
+
+void RKWardMainWindow::configureCarbonCopy () {
+	RK_TRACE (APP);
+
+	KDialog *dialog = new KDialog ();
+	dialog->setCaption (i18n ("Carbon Copy Settings"));
+	RKCarbonCopySettings *settings = new RKCarbonCopySettings (dialog);
+	dialog->setMainWidget (settings);
+	dialog->setButtons (KDialog::Ok | KDialog::Apply | KDialog::Cancel);
+	dialog->setAttribute (Qt::WA_DeleteOnClose);
+	connect (dialog, SIGNAL (okClicked()), settings, SLOT (applyChanges ())); 
+	connect (dialog, SIGNAL (applyClicked()), settings, SLOT (applyChanges ())); 
+	dialog->show ();
+}
+
+void RKWardMainWindow::initToolViewsAndR () {
+	RK_TRACE (APP);
+
+	RObjectBrowser::object_browser = new RObjectBrowser (0, true);
+	RObjectBrowser::mainBrowser ()->setCaption (i18n ("Workspace"));
+	RKToolWindowList::registerToolWindow (RObjectBrowser::mainBrowser (), "workspace", RKToolWindowList::Left, Qt::AltModifier + Qt::Key_1);
+
+	RKCommandLog::rkcommand_log = new RKCommandLog (0, true);
+	RKToolWindowList::registerToolWindow (RKCommandLog::rkcommand_log, "commandlog", RKToolWindowList::Bottom, Qt::AltModifier + Qt::Key_3);
+
+	startR ();
+
+	RKFileBrowser::main_browser = new RKFileBrowser (0, true);
+	RKFileBrowser::main_browser->setCaption (i18n ("Files"));
+	RKToolWindowList::registerToolWindow (RKFileBrowser::main_browser, "filebrowser", RKToolWindowList::Left, Qt::AltModifier + Qt::Key_2);
+
+	RControlWindow::control_window = new RControlWindow (0, true);
+	RControlWindow::getControl ()->setCaption (i18n ("Pending Jobs"));
+	RKToolWindowList::registerToolWindow (RControlWindow::getControl (), "pendingjobs", RKToolWindowList::Nowhere, Qt::AltModifier + Qt::Key_4);
+
+	RKConsole *console = new RKConsole (0, true);
+	RKConsole::setMainConsole (console);
+	RKToolWindowList::registerToolWindow (console, "console", RKToolWindowList::Bottom, Qt::AltModifier + Qt::Key_5);
+
+	RKHelpSearchWindow *help_search = new RKHelpSearchWindow (0, true);
+	RKHelpSearchWindow::main_help_search = help_search;
+	RKToolWindowList::registerToolWindow (help_search, "helpsearch", RKToolWindowList::Bottom, Qt::AltModifier + Qt::Key_6);
+
+	RKWorkplace::mainWorkplace ()->placeToolWindows ();
+}
+
+void RKWardMainWindow::initActions() {  
 	RK_TRACE (APP);
 	KAction *action;
 
@@ -321,9 +358,10 @@ void RKWardMainWindow::initActions()
 	new_command_editor->setIcon (RKStandardIcons::getIcon (RKStandardIcons::WindowCommandEditor));
 
 	fileOpen = actionCollection ()->addAction (KStandardAction::Open, "file_openy", this, SLOT(slotOpenCommandEditor()));
-	fileOpen->setText (i18n ("Open R Script File"));
+	fileOpen->setText (i18n ("Open R Script File..."));
 
 	fileOpenRecent = static_cast<KRecentFilesAction*> (actionCollection ()->addAction (KStandardAction::OpenRecent, "file_open_recenty", this, SLOT(slotOpenCommandEditor (const KUrl&))));
+	fileOpenRecent->setText (i18n ("Open Recent R Script File"));
 
 #ifdef Q_WS_WIN
 	// TODO: find the cause and fix it! http://sourceforge.net/tracker/?func=detail&aid=2848341&group_id=50231&atid=459007
@@ -335,11 +373,12 @@ void RKWardMainWindow::initActions()
 #endif
 
 	fileOpenWorkspace = actionCollection ()->addAction (KStandardAction::Open, "file_openx", this, SLOT(slotFileOpenWorkspace()));
-	fileOpenWorkspace->setText (i18n ("Open Workspace"));
+	fileOpenWorkspace->setText (i18n ("Open Workspace..."));
 	fileOpenWorkspace->setShortcut (Qt::ControlModifier + Qt::ShiftModifier + Qt::Key_O);
 	fileOpenWorkspace->setStatusTip (i18n ("Opens an existing document"));
 
 	fileOpenRecentWorkspace = static_cast<KRecentFilesAction*> (actionCollection ()->addAction (KStandardAction::OpenRecent, "file_open_recentx", this, SLOT(slotFileOpenRecentWorkspace(const KUrl&))));
+	fileOpenRecentWorkspace->setText (i18n ("Open Recent Workspace"));
 	fileOpenRecentWorkspace->setStatusTip (i18n ("Opens a recently used file"));
 
 	fileSaveWorkspace = actionCollection ()->addAction (KStandardAction::Save, "file_savex", this, SLOT(slotFileSaveWorkspace()));
@@ -354,6 +393,15 @@ void RKWardMainWindow::initActions()
 	fileQuit = actionCollection ()->addAction (KStandardAction::Quit, "file_quitx", this, SLOT(close()));
 	fileQuit->setStatusTip (i18n ("Quits the application"));
 
+	interrupt_all_commands = actionCollection ()->addAction ("cancel_all_commands", this, SLOT (slotCancelAllCommands()));
+	interrupt_all_commands->setText (i18n ("Interrupt all commands"));
+	interrupt_all_commands->setShortcut (Qt::ShiftModifier + Qt::Key_Escape);
+	interrupt_all_commands->setIcon (RKStandardIcons::getIcon (RKStandardIcons::ActionInterrupt));
+	interrupt_all_commands->setEnabled (false);		// enabled from within setRStatus()
+
+	action = actionCollection ()->addAction ("carbon_copy", this, SLOT (configureCarbonCopy ()));
+	action->setText (i18n ("CC commands to output..."));
+
 	// These two currently do the same thing
 	action = actionCollection ()->addAction ("load_unload_libs", this, SLOT (slotFileLoadLibs()));
 	action->setText (i18n ("Load / Unload Packages"));
@@ -388,14 +436,76 @@ void RKWardMainWindow::initActions()
 	view_menu_dummy = actionCollection ()->addAction ("view_menu_dummy", this);
 	view_menu_dummy->setText (edit_menu_dummy->text ());
 	view_menu_dummy->setEnabled (false);
-	run_menu_dummy = actionCollection ()->addAction ("run_menu_dummy", this);
-	run_menu_dummy->setText (edit_menu_dummy->text ());
-	run_menu_dummy->setEnabled (false);
+
+	// collections for the toolbar:
+	KActionMenu* open_any_action = new KActionMenu (KIcon ("document-open-folder"), i18n ("Open..."), this);
+	open_any_action->setDelayed (false);
+	actionCollection ()->addAction ("open_any", open_any_action);
+
+	open_any_action->addAction (fileOpenWorkspace);
+	open_any_action->addAction (fileOpenRecentWorkspace);
+	open_any_action->addSeparator ();
+	open_any_action->addAction (fileOpen);
+	open_any_action->addAction (fileOpenRecent);
+	open_any_action->addSeparator ();
+	proxy_import = new KAction (i18n ("Import"), this);
+	open_any_action->addAction (proxy_import);
+
+	KActionMenu* new_any_action = new KActionMenu (KIcon ("document-new"), i18n ("Create..."), this);
+	new_any_action->setDelayed (false);
+	actionCollection ()->addAction ("new_any", new_any_action);
+
+	new_any_action->addAction (new_data_frame);
+	new_any_action->addAction (new_command_editor);
+
+	save_any_action = new KActionMenu (KIcon ("document-save"), i18n ("Save..."), this);
+	save_any_action->setDelayed (false);
+	actionCollection ()->addAction ("save_any", save_any_action);
+
+	proxy_export = new KAction (i18n ("Export"), this);
+	save_any_action->addAction (fileSaveWorkspace);
+	save_any_action->addAction (fileSaveWorkspaceAs);
+	save_any_action->addSeparator ();
+// TODO: A way to add R-script-save actions, dynamically, would be nice
+	save_actions_plug_point = save_any_action->addSeparator ();
+	save_any_action->addAction (proxy_export);
+}
+
+/*
+// debug code: prints out all current actions
+void printActionsRecursive (QAction* action, const QString &prefix) {
+	if (action->menu ()) {
+		foreach (QAction *a, action->menu ()->actions ()) printActionsRecursive (a, prefix + action->text () + "->");
+	} else {
+		qDebug ("%s", qPrintable (prefix + action->text ()));
+	}
+}
+*/
+
+void updateEmptyMenuIndicator (QAction* indicator, const QMenu *menu) {
+	if (!menu) {
+		indicator->setVisible (false);
+		return;
+	}
+
+	// NOTE: QMenu::isEmpty () does not work, here
+	QList<QAction *> actions = menu->actions ();
+	for (int i = 0; i < actions.size (); ++i) {
+		if (actions[i] == indicator) continue;
+		if (actions[i]->isSeparator ()) continue;
+		if (actions[i]->isVisible ()) {
+			indicator->setVisible (false);
+			return;
+		}
+	}
+
+	indicator->setVisible (true);
 }
 
 void RKWardMainWindow::partChanged (KParts::Part *part) {
 	RK_TRACE (APP);
 
+	if (gui_rebuild_locked) return;
 	createGUI (part);
 
 	if (!guiFactory ()) {
@@ -403,12 +513,43 @@ void RKWardMainWindow::partChanged (KParts::Part *part) {
 		return;
 	}
 
-	QMenu* menu = dynamic_cast<QMenu*>(guiFactory ()->container ("edit", this));
-	edit_menu_dummy->setVisible (menu && (menu->isEmpty ()));
-	menu = dynamic_cast<QMenu*>(guiFactory ()->container ("view", this));
-	view_menu_dummy->setVisible (menu && (menu->isEmpty ()));
-	menu = dynamic_cast<QMenu*>(guiFactory ()->container ("run", this));
-	run_menu_dummy->setVisible (menu && (menu->isEmpty ()));
+	updateEmptyMenuIndicator (edit_menu_dummy, dynamic_cast<QMenu*>(guiFactory ()->container ("edit", this)));
+	updateEmptyMenuIndicator (view_menu_dummy, dynamic_cast<QMenu*>(guiFactory ()->container ("view", this)));
+
+	// plug save file actions into the toolbar collections
+	RK_ASSERT (save_any_action);
+	for (int i = 0; i < plugged_save_actions.size (); ++i) {
+		QAction* a = plugged_save_actions[i].data ();
+		if (a) save_any_action->removeAction (a);
+	}
+	plugged_save_actions.clear ();
+
+	RKMDIWindow *w = RKWorkplace::mainWorkplace ()->activeWindow (RKMDIWindow::Attached);
+	if (w && (w->isType (RKMDIWindow::CommandEditorWindow))) {
+		QAction *a = static_cast<RKCommandEditorWindow*>(w)->fileSaveAction ();
+		if (a) plugged_save_actions.append (a);
+		a = static_cast<RKCommandEditorWindow*>(w)->fileSaveAsAction ();
+		if (a) plugged_save_actions.append (a);
+	}
+	for (int i = 0; i < plugged_save_actions.size (); ++i) {
+		save_any_action->insertAction (save_actions_plug_point, plugged_save_actions[i]);
+	}
+/*
+	// debug code: prints out all current actions
+	foreach (QAction *action, menuBar ()->actions ()) printActionsRecursive (action, QString ());
+*/
+}
+
+void RKWardMainWindow::lockGUIRebuild (bool lock) {
+	RK_TRACE (APP);
+
+	if (lock) {
+		RK_ASSERT (!gui_rebuild_locked);
+		gui_rebuild_locked = true;
+	} else {
+		gui_rebuild_locked = false;
+		partChanged (part_manager->activePart ());
+	}
 }
 
 void RKWardMainWindow::initStatusBar () {
@@ -418,12 +559,21 @@ void RKWardMainWindow::initStatusBar () {
 	statusBar ()->addWidget (statusbar_ready);
 	statusbar_cwd = new KSqueezedTextLabel (statusBar ());
 	statusbar_cwd->setAlignment (Qt::AlignRight);
+	statusbar_cwd->setToolTip (i18n ("Current working directory"));
 	statusBar ()->addWidget (statusbar_cwd, 10);
 	updateCWD ();
 
-	statusbar_r_status = new QLabel (statusBar ());
+	KHBox *box = new KHBox (statusBar ());
+	box->setSpacing (0);
+	statusbar_r_status = new QLabel ("&nbsp;<b>R</b>&nbsp;", box);
 	statusbar_r_status->setFixedHeight (statusBar ()->fontMetrics ().height () + 2);
-	statusBar ()->addPermanentWidget (statusbar_r_status, 0);
+
+	QToolButton* dummy = new QToolButton (box);
+	dummy->setDefaultAction (interrupt_all_commands);
+	dummy->setFixedHeight (statusbar_r_status->height ());
+	dummy->setAutoRaise (true);
+
+	statusBar ()->addPermanentWidget (box, 0);
 	setRStatus (Starting);
 }
 
@@ -489,7 +639,7 @@ bool RKWardMainWindow::doQueryQuit () {
 	slotSetStatusBarText (i18n ("Exiting..."));
 	saveOptions ();
 	if (RKSettingsModuleGeneral::workplaceSaveMode () == RKSettingsModuleGeneral::SaveWorkplaceWithSession) {
-		RKSettingsModuleGeneral::setSavedWorkplace (RKWorkplace::mainWorkplace ()->makeWorkplaceDescription ("\n", false), KGlobal::config ().data ());
+		RKSettingsModuleGeneral::setSavedWorkplace (RKWorkplace::mainWorkplace ()->makeWorkplaceDescription ().join ("\n"), KGlobal::config ().data ());
 	}
 
 //	if (!RObjectList::getGlobalEnv ()->isEmpty ()) {
@@ -504,14 +654,17 @@ bool RKWardMainWindow::doQueryQuit () {
 //	}
 
 	RKWorkplace::RKWorkplaceObjectList map = RKWorkplace::mainWorkplace ()->getObjectList ();
-	for (RKWorkplace::RKWorkplaceObjectList::const_iterator it = map.constBegin (); it != map.constEnd (); ++it){
+	for (RKWorkplace::RKWorkplaceObjectList::const_iterator it = map.constBegin (); it != map.constEnd (); ++it) {
+		lockGUIRebuild (true);
 		if (!(*it)->close (true)) {
 			if (!(*it)->isType (RKMDIWindow::X11Window)) {	// X11 windows have a delayed close
 				// If a child refuses to close, we return false.
 				slotSetStatusReady ();
+				lockGUIRebuild (false);
 				return false;
 			}
 		}
+		lockGUIRebuild (false);
 	}
 
 	return true;
@@ -647,14 +800,16 @@ void RKWardMainWindow::setRStatus (RStatus status) {
 
 	QColor status_color;
 	if (status == Busy) {
-		statusbar_r_status->setText (i18n ("R engine busy"));
 		status_color = QColor (255, 0, 0);
+		statusbar_r_status->setToolTip (i18n ("The <b>R</b> engine is busy."));
+		interrupt_all_commands->setEnabled (true);
 	} else if (status == Idle) {
-		statusbar_r_status->setText (i18n ("R engine idle"));
 		status_color = QColor (0, 255, 0);
+		statusbar_r_status->setToolTip (i18n ("The <b>R</b> engine is idle."));
+		interrupt_all_commands->setEnabled (false);
 	} else {
-		statusbar_r_status->setText (i18n ("R engine starting"));
 		status_color = QColor (255, 255, 0);
+		statusbar_r_status->setToolTip (i18n ("The <b>R</b> engine is being initialized."));
 	}
 	QPalette palette = statusbar_r_status->palette ();
 	palette.setBrush (statusbar_r_status->backgroundRole(), QBrush (status_color));
