@@ -2,7 +2,7 @@
                           rksettingsmoduleplugins  -  description
                              -------------------
     begin                : Wed Jul 28 2004
-    copyright            : (C) 2004, 2006, 2007 by Thomas Friedrichsmeier
+    copyright            : (C) 2004, 2006, 2007, 2010 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -52,9 +52,10 @@ public:
 	static bool showCodeByDefault () { return show_code; };
 	static int defaultCodeHeight () { return code_size; };
 public slots:
-	void pathsChanged ();
-	void settingChanged (int);
+	void settingChanged ();
 	void browseRequest (QStringList* strings);
+	/** Invoke the plugin download dialog. */
+	void downloadPlugins ();
 private:
 	MultiStringSelector *map_choser;
 	QButtonGroup *button_group;
@@ -65,6 +66,11 @@ private:
 	static PluginPrefs interface_pref;
 	static bool show_code;
 	static int code_size;
+
+	static void installPluginPack (const QString &archive_file);
+	static void uninstallPluginPack (const QString &archive_file);
+	static QString baseNameOfPluginPack (const QString &archive_file);
+	static QStringList findPluginMapsRecursive (const QString &basedir);
 };
 
 #endif
