@@ -2,7 +2,7 @@
                           rkcommandeditorwindow  -  description
                              -------------------
     begin                : Mon Aug 30 2004
-    copyright            : (C) 2004, 2006, 2007, 2009, 2010 by Thomas Friedrichsmeier
+    copyright            : (C) 2004, 2006, 2007, 2009, 2010, 2011 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -56,6 +56,7 @@
 #include "../misc/rkxmlguisyncer.h"
 #include "../misc/rkjobsequence.h"
 #include "../core/robjectlist.h"
+#include "../rbackend/rinterface.h"
 #include "../settings/rksettings.h"
 #include "../settings/rksettingsmodulecommandeditor.h"
 #include "../rkconsole.h"
@@ -128,7 +129,7 @@ RKCommandEditorWindow::RKCommandEditorWindow (QWidget *parent, bool use_r_highli
 	cc_iface = 0;
 	hinter = 0;
 	if (use_r_highlighting) {
-		setRHighlighting ();
+		RKCommandHighlighter::setHighlighting (m_doc, RKCommandHighlighter::RScript);
 		cc_iface = qobject_cast<KTextEditor::CodeCompletionInterface*> (m_view);
 		if (cc_iface) {
 			cc_iface->setAutomaticInvocationEnabled (true);
@@ -158,6 +159,7 @@ RKCommandEditorWindow::~RKCommandEditorWindow () {
 	RK_TRACE (COMMANDEDITOR);
 	delete hinter;
 	delete m_doc;
+	if (!delete_on_close.isEmpty ()) KIO::del (delete_on_close)->start ();
 }
 
 void RKCommandEditorWindow::fixupPartGUI () {
@@ -168,9 +170,23 @@ void RKCommandEditorWindow::fixupPartGUI () {
 	RKCommonFunctions::moveContainer (m_view, "Menu", "tools", "edit", true);
 }
 
+QAction *findAction (KTextEditor::View* view, const QString &actionName) {
+	// katepart has more than one actionCollection
+	QList<KActionCollection*> acs = view->findChildren<KActionCollection*>();
+	acs.append (view->actionCollection ());
+
+	foreach (KActionCollection* ac, acs) {
+		QAction* found = ac->action (actionName);
+		if (found) return found;
+	}
+
+	return 0;
+}
+
 void RKCommandEditorWindow::initializeActions (KActionCollection* ac) {
 	RK_TRACE (COMMANDEDITOR);
 
+	RKStandardActions::copyLinesToOutput (this, this, SLOT (copyLinesToOutput()));
 	RKStandardActions::pasteSpecial (this, this, SLOT (paste(const QString&)));
 
 	action_run_all = RKStandardActions::runAll (this, this, SLOT (runAll()));
@@ -197,6 +213,11 @@ void RKCommandEditorWindow::initializeActions (KActionCollection* ac) {
 	action_setwd_to_script->setHelpText (i18n ("Change the working directory to the directory of this script"));
 #endif
 	action_setwd_to_script->setIcon (RKStandardIcons::getIcon (RKStandardIcons::ActionCDToScript));
+
+	file_save = findAction (m_view, "file_save");
+	if (file_save) file_save->setText (i18n ("Save Script..."));
+	file_save_as = findAction (m_view, "file_save_as");
+	if (file_save_as) file_save_as->setText (i18n ("Save Script As..."));
 }
 
 void RKCommandEditorWindow::initBlocks () {
@@ -292,12 +313,6 @@ QString RKCommandEditorWindow::fullCaption () {
 	}
 }
 
-QString RKCommandEditorWindow::getDescription () {
-	RK_TRACE (COMMANDEDITOR);
-
-	return ("script:" + m_doc->url ().url ());
-}
-
 void RKCommandEditorWindow::closeEvent (QCloseEvent *e) {
 	if (isModified ()) {
 		int status = KMessageBox::warningYesNo (this, i18n ("The document \"%1\" has been modified. Close it anyway?", windowTitle ()), i18n ("File not saved"));
@@ -311,14 +326,6 @@ void RKCommandEditorWindow::closeEvent (QCloseEvent *e) {
 	QWidget::closeEvent (e);
 }
 
-// KDE4 TODO: inline
-void RKCommandEditorWindow::setRHighlighting () {
-	RK_TRACE (COMMANDEDITOR);
-
-	// set syntax-highlighting for R
-	if (!m_doc->setHighlightingMode("R Script")) RK_DO (qDebug ("R syntax highlighting defintion not found!"), COMMANDEDITOR, DL_ERROR);
-}
-
 void RKCommandEditorWindow::copy () {
 	RK_TRACE (COMMANDEDITOR);
 
@@ -331,17 +338,25 @@ void RKCommandEditorWindow::setReadOnly (bool ro) {
 	m_doc->setReadWrite (!ro);
 }
 
-bool RKCommandEditorWindow::openURL (const KUrl &url, const QString& encoding, bool use_r_highlighting, bool read_only){
+bool RKCommandEditorWindow::openURL (const KUrl &url, const QString& encoding, bool use_r_highlighting, bool read_only, bool delete_on_close){
 	RK_TRACE (COMMANDEDITOR);
 
 	// encoding must be set *before* loading the file
 	if (!encoding.isEmpty ()) m_doc->setEncoding (encoding);
 	if (m_doc->openUrl (url)){
-		if (use_r_highlighting) setRHighlighting ();
+		if (use_r_highlighting) RKCommandHighlighter::setHighlighting (m_doc, RKCommandHighlighter::RScript);
 		setReadOnly (read_only);
 
 		updateCaption ();
 
+		if (delete_on_close) {
+			if (!read_only) {
+				RK_ASSERT (false);
+				return true;
+			}
+			RKCommandEditorWindow::delete_on_close=url;
+		}
+
 		return true;
 	}
 	return false;
@@ -513,6 +528,16 @@ QString RKCommandEditorWindow::currentCompletionWord () const {
 	return RKCommonFunctions::getCurrentSymbol (current_line, cursor_pos, false);
 }
 
+#if KDE_IS_VERSION(4,2,0)
+KTextEditor::Range RKCodeCompletionModel::completionRange (KTextEditor::View *view, const KTextEditor::Cursor &position) {
+	QString current_line = view->document ()->line (position.line ());
+	int start;
+	int end;
+	RKCommonFunctions::getCurrentSymbolOffset (current_line, position.column (), false, &start, &end);
+	return KTextEditor::Range (position.line (), start, position.line (), end);
+}
+#endif
+
 void RKCommandEditorWindow::tryCompletion () {
 	// TODO: merge this with RKConsole::doTabCompletion () somehow
 	RK_TRACE (COMMANDEDITOR);
@@ -546,22 +571,17 @@ void RKCommandEditorWindow::tryCompletion () {
 	}
 }
 
-bool RKCommandEditorWindow::provideContext (unsigned int line_rev, QString *context, int *cursor_position) {
+QString RKCommandEditorWindow::provideContext (int line_rev) {
 	RK_TRACE (COMMANDEDITOR);
 
 	KTextEditor::Cursor c = m_view->cursorPosition();
-	uint current_line_num=c.line(); uint cursor_pos=c.column();
+	int current_line_num=c.line(); int cursor_pos=c.column();
 
-	if (line_rev > current_line_num) return false;
+	if (line_rev > current_line_num) return QString ();
 
-	if (line_rev == 0) {
-		*cursor_position = cursor_pos;
-	} else {
-		*cursor_position = -1;
-	}
-	*context = m_doc->line (current_line_num - line_rev);
-
-	return true;
+	QString ret = m_doc->line (current_line_num - line_rev);
+	if (line_rev == 0) ret = ret.left (cursor_pos);
+	return ret;
 }
 
 void RKCommandEditorWindow::paste (const QString& text) {
@@ -579,7 +599,7 @@ void RKCommandEditorWindow::setWDToScript () {
 	// KURL::directory () returns a leading slash on windows as of KDElibs 4.3
 	while (dir.startsWith ('/')) dir.remove (0, 1);
 #endif
-	RKConsole::pipeUserCommand (new RCommand ("setwd (\"" + dir + "\")", RCommand::User, i18n ("cd to current script directory")));
+	RKConsole::pipeUserCommand ("setwd (\"" + dir + "\")");
 }
 
 void RKCommandEditorWindow::runSelection() {
@@ -588,7 +608,7 @@ void RKCommandEditorWindow::runSelection() {
 	QString command = m_view->selectionText ();
 	if (command.isEmpty ()) return;
 
-	RKConsole::pipeUserCommand (new RCommand (command, RCommand::User, QString::null));
+	RKConsole::pipeUserCommand (command);
 }
 
 void RKCommandEditorWindow::runLine() {
@@ -596,13 +616,18 @@ void RKCommandEditorWindow::runLine() {
 
 	KTextEditor::Cursor c = m_view->cursorPosition();
 	QString command = m_doc->line (c.line());
-	if (!command.isEmpty ()) RKConsole::pipeUserCommand (new RCommand (command, RCommand::User, QString::null));
+	if (!command.isEmpty ()) RKConsole::pipeUserCommand (command + '\n');
 
 	// advance to next line (NOTE: m_view->down () won't work on auto-wrapped lines)
 	c.setLine(c.line() + 1);
 	m_view->setCursorPosition (c);
 }
 
+void RKCommandEditorWindow::copyLinesToOutput () {
+	RK_TRACE (COMMANDEDITOR);
+
+	RKCommandHighlighter::copyLinesToOutput (m_view, RKCommandHighlighter::RScript);
+}
 
 void RKCommandEditorWindow::runAll () {
 	RK_TRACE (COMMANDEDITOR);
@@ -629,7 +654,7 @@ void RKCommandEditorWindow::runBlock () {
 		QString command = m_doc->text (*(block_records[index].range));
 		if (command.isEmpty ()) return;
 	
-		RKConsole::pipeUserCommand (new RCommand (command, RCommand::User, QString::null));
+		RKConsole::pipeUserCommand (command);
 	}
 }
 
@@ -750,6 +775,8 @@ void RKCommandEditorWindow::selectionChanged (KTextEditor::View* view) {
 
 //////////////////////// RKFunctionArgHinter //////////////////////////////
 
+#include <QToolTip>
+
 #include "../core/rfunctionobject.h"
 
 RKFunctionArgHinter::RKFunctionArgHinter (RKScriptContextProvider *provider, KTextEditor::View* view) {
@@ -764,14 +791,15 @@ RKFunctionArgHinter::RKFunctionArgHinter (RKScriptContextProvider *provider, KTe
 		obj->installEventFilter (this);
 	}
 
-	arghints_popup = new QFrame (0, Qt::ToolTip);
-	QVBoxLayout *layout = new QVBoxLayout (arghints_popup);
-	layout->setContentsMargins (2, 2, 2, 2);
-	arghints_popup->setFrameStyle (QFrame::Plain);
-	arghints_popup->setFrameShape (QFrame::Box);
+	arghints_popup = new QLabel (0, Qt::ToolTip);
+	arghints_popup->setMargin (2);
+	QPalette p = QToolTip::palette ();		// HACK to trick the style into using the correct color
+	p.setColor (QPalette::Inactive, QPalette::Window, p.color (QPalette::Inactive, QPalette::ToolTipBase));
+	p.setColor (QPalette::Inactive, QPalette::Text, p.color (QPalette::Inactive, QPalette::ToolTipText));
+	arghints_popup->setPalette (p);
+	arghints_popup->setFrameStyle (QFrame::Box);
 	arghints_popup->setLineWidth (1);
-	arghints_popup_text = new QLabel (arghints_popup);
-	layout->addWidget (arghints_popup_text);
+	arghints_popup->setWordWrap (true);
 	arghints_popup->hide ();
 	active = false;
 
@@ -780,6 +808,7 @@ RKFunctionArgHinter::RKFunctionArgHinter (RKScriptContextProvider *provider, KTe
 
 RKFunctionArgHinter::~RKFunctionArgHinter () {
 	RK_TRACE (COMMANDEDITOR);
+	delete arghints_popup;
 }
 
 void RKFunctionArgHinter::tryArgHint () {
@@ -794,68 +823,42 @@ void RKFunctionArgHinter::tryArgHint () {
 void RKFunctionArgHinter::tryArgHintNow () {
 	RK_TRACE (COMMANDEDITOR);
 
-	int line_rev;
-	int cursor_pos;
-	QString current_context;
-	QString current_line;
-
-	// fetch the most immediate context line. More will be fetched later, if appropriate
-	bool have_context = provider->provideContext (line_rev = 0, &current_line, &cursor_pos);
-	RK_ASSERT (have_context);
-	RK_ASSERT (cursor_pos >= 0);
-	current_context = current_line;
-
-	// find the corrresponding opening brace
-	int matching_left_brace_pos;
+	// find the active opening brace
+	int line_rev = -1;
 	int brace_level = 1;
-	int i = cursor_pos;
+	int potential_symbol_end = -1;
+	QString full_context;
+	while (potential_symbol_end < 0) {
+		QString context_line = provider->provideContext (++line_rev);
+		if (context_line.isNull ()) break;
 
-	// fix up seems to be needed
-	if (current_context.isEmpty ()) {
-		hideArgHint ();
-		return;
-	}
-	if (i >= current_context.size ()) i = current_context.size () -1;
-	if (i < 0) i = 0;
-
-	while (true) {
-		if (current_context.at (i) == QChar (')')) {
-			brace_level++;
-		} else if (current_context.at (i) == QChar ('(')) {
-			brace_level--;
-			if (!brace_level) break;
-		}
-
-		--i;
-		if (i < 0) {
-			bool have_context = provider->provideContext (++line_rev, &current_line, &cursor_pos);
-			if ((!have_context) || (current_line.isEmpty ())) break;
-
-			RK_ASSERT (cursor_pos < 0);
-			current_context.prepend (current_line);
-			i = current_line.length () - 1;
+		full_context.prepend (context_line);
+		int pos = context_line.length ();
+		while (--pos >= 0) {
+			QChar c = full_context.at (pos);
+			if (c == ')') ++brace_level;
+			else if (c == '(') {
+				--brace_level;
+				if (brace_level == 0) {
+					potential_symbol_end = pos - 1;
+					break;
+				}
+			}
 		}
 	}
 
-	if (!brace_level) matching_left_brace_pos = i;
-	else {
-		hideArgHint ();
-		return;
-	}
-
-	// now find where the symbol to the left ends
+	// now find out where the symbol to the left of the opening brace ends
 	// there cannot be a line-break between the opening brace, and the symbol name (or can there?), so no need to fetch further context
-	int potential_symbol_end = matching_left_brace_pos - 1;
-	while ((potential_symbol_end >= 0) && current_context.at (potential_symbol_end).isSpace ()) {
+	while ((potential_symbol_end >= 0) && full_context.at (potential_symbol_end).isSpace ()) {
 		--potential_symbol_end;
 	}
-	if (potential_symbol_end < 0) {
+	if (potential_symbol_end <= 0) {
 		hideArgHint ();
 		return;
 	}
 
 	// now identify the symbol and object (if any)
-	QString effective_symbol = RKCommonFunctions::getCurrentSymbol (current_context, potential_symbol_end+1);
+	QString effective_symbol = RKCommonFunctions::getCurrentSymbol (full_context, potential_symbol_end+1);
 	if (effective_symbol.isEmpty ()) {
 		hideArgHint ();
 		return;
@@ -868,8 +871,8 @@ void RKFunctionArgHinter::tryArgHintNow () {
 	}
 
 	// initialize and show popup
-	arghints_popup_text->setText (effective_symbol + " (" + static_cast<RFunctionObject*> (object)->printArgs () + ')');
-	arghints_popup->resize (arghints_popup_text->sizeHint () + QSize (2, 2));
+	arghints_popup->setText (effective_symbol + " (" + static_cast<RFunctionObject*> (object)->printArgs () + ')');
+	arghints_popup->resize (arghints_popup->sizeHint () + QSize (2, 2));
 	active = true;
 	updater.start (50);
 	updateArgHintWindow ();
@@ -880,7 +883,7 @@ void RKFunctionArgHinter::updateArgHintWindow () {
 
 	if (!active) return;
 
-	arghints_popup->move (view->mapToGlobal (view->cursorPositionCoordinates () + QPoint (0, arghints_popup->height ())));
+	arghints_popup->move (view->mapToGlobal (view->cursorPositionCoordinates () + QPoint (0, arghints_popup->fontMetrics ().lineSpacing () + arghints_popup->margin ()*2)));
 	if (view->hasFocus ()) arghints_popup->show ();
 	else arghints_popup->hide ();
 }
@@ -903,9 +906,7 @@ bool RKFunctionArgHinter::eventFilter (QObject *, QEvent *e) {
 			tryArgHint ();
 		} else {
 			QString text = k->text ();
-			if (text == "(") {
-				tryArgHint ();
-			} else if (text == ")") {
+			if ((text == "(") || (text == ")") || (text == ",")) {
 				tryArgHint ();
 			}
 		}
@@ -999,4 +1000,187 @@ QVariant RKCodeCompletionModel::data (const QModelIndex& index, int role) const
 	return QVariant ();
 }
 
+
+
+// static
+KTextEditor::Document* RKCommandHighlighter::_doc = 0;
+KTextEditor::Document* RKCommandHighlighter::getDoc () {
+	if (_doc) return _doc;
+
+	RK_TRACE (COMMANDEDITOR);
+	KTextEditor::Editor* editor = KTextEditor::editor("katepart");
+	RK_ASSERT (editor);
+
+	_doc = editor->createDocument (RKWardMainWindow::getMain ());
+// NOTE: In KDE 4.4.5, a (dummy) view is needed to access highlighting attributes. According to a katepart error message, this will be fixed, eventually.
+// TODO: check whether this is fixed in some later version of KDE
+	QWidget* view = _doc->createView (0);
+	view->hide ();
+	RK_ASSERT (_doc);
+	return _doc;
+}
+
+#if KDE_IS_VERSION(4,4,0)
+#	include <ktexteditor/highlightinterface.h>
+#include <QTextDocument>
+
+//////////
+// NOTE: Most of the exporting code is copied from the katepart HTML exporter plugin more or less verbatim! (Source license: LGPL v2)
+//////////
+QString exportText(const QString& text, const KTextEditor::Attribute::Ptr& attrib, const KTextEditor::Attribute::Ptr& m_defaultAttribute) {
+	if ( !attrib || !attrib->hasAnyProperty() || attrib == m_defaultAttribute ) {
+		return (Qt::escape(text));
+	}
+
+	QString ret;
+	if ( attrib->fontBold() ) {
+		ret.append ("<b>");
+	}
+	if ( attrib->fontItalic() ) {
+		ret.append ("<i>");
+	}
+
+	bool writeForeground = attrib->hasProperty(QTextCharFormat::ForegroundBrush)
+		&& (!m_defaultAttribute || attrib->foreground().color() != m_defaultAttribute->foreground().color());
+	bool writeBackground = attrib->hasProperty(QTextCharFormat::BackgroundBrush)
+		&& (!m_defaultAttribute || attrib->background().color() != m_defaultAttribute->background().color());
+
+	if ( writeForeground || writeBackground ) {
+		ret.append (QString("<span style='%1%2'>")
+					.arg(writeForeground ? QString(QLatin1String("color:") + attrib->foreground().color().name() + QLatin1Char(';')) : QString())
+					.arg(writeBackground ? QString(QLatin1String("background:") + attrib->background().color().name() + QLatin1Char(';')) : QString()));
+	}
+
+	ret.append (Qt::escape(text));
+
+	if ( writeBackground || writeForeground ) {
+		ret.append ("</span>");
+	}
+	if ( attrib->fontItalic() ) {
+		ret.append ("</i>");
+	}
+	if ( attrib->fontBold() ) {
+		ret.append ("</b>");
+	}
+
+	return ret;
+}
+
+QString RKCommandHighlighter::commandToHTML (const QString r_command, HighlightingMode mode) {
+	KTextEditor::Document* doc = getDoc ();
+	KTextEditor::HighlightInterface *iface = qobject_cast<KTextEditor::HighlightInterface*> (_doc);
+	RK_ASSERT (iface);
+	if (!iface) return (QString ("<pre>") + r_command + "</pre>");
+
+	doc->setText (r_command);
+	if (r_command.endsWith ("\n")) doc->removeLine (doc->lines () - 1);
+	setHighlighting (doc, mode);
+	QString ret;
+
+	QString opening;
+	KTextEditor::Attribute::Ptr m_defaultAttribute = iface->defaultStyle(KTextEditor::HighlightInterface::dsNormal);
+	if ( !m_defaultAttribute ) {
+		opening = "<pre class=\"%3\">";
+	} else {
+		opening = QString("<pre style='%1%2' class=\"%3\">")
+				.arg(m_defaultAttribute->fontBold() ? "font-weight:bold;" : "")
+				.arg(m_defaultAttribute->fontItalic() ? "text-style:italic;" : "");
+				// Note: copying the default text/background colors is pointless in our case, and leads to subtle inconsistencies.
+	}
+
+	const KTextEditor::Attribute::Ptr noAttrib(0);
+
+	if (mode == RScript) ret = opening.arg ("code");
+	enum {
+		Command,
+		Output,
+		Warning,
+		None
+	} previous_chunk = None;
+	for (int i = 0; i < doc->lines (); ++i)
+	{
+		const QString &line = doc->line(i);
+		QList<KTextEditor::HighlightInterface::AttributeBlock> attribs = iface->lineAttributes(i);
+		int lineStart = 0;
+
+		if (mode == RInteractiveSession) {
+			if (line.startsWith ("> ") || line.startsWith ("+ ")) {
+				lineStart = 2;	// skip the prompt. Prompt will be indicated by the CSS, instead
+				if (previous_chunk != Command) {
+					if (previous_chunk != None) ret.append ("</pre>");
+					ret.append (opening.arg ("code"));
+					previous_chunk = Command;
+				}
+			} else {
+				if (previous_chunk != Output) {
+					if (previous_chunk != None) ret.append ("</pre>");
+					ret.append (opening.arg ("output_normal"));
+					previous_chunk = Output;
+				}
+				ret.append (Qt::escape (line) + "\n");	// don't copy output "highlighting". It is set using CSS, instead
+				continue;
+			}
+		}
+
+		int handledUntil = lineStart;
+		int remainingChars = line.length() - handledUntil;
+		foreach ( const KTextEditor::HighlightInterface::AttributeBlock& block, attribs ) {
+			if ((block.start + block.length) <= handledUntil) continue;
+			int start = qMax(block.start, lineStart);
+			if ( start > handledUntil ) {
+				ret += exportText( line.mid( handledUntil, start - handledUntil ), noAttrib, m_defaultAttribute );
+			}
+			int length = qMin(block.length, remainingChars);
+			ret += exportText( line.mid( start, length ), block.attribute, m_defaultAttribute);
+			handledUntil = start + length;
+		}
+
+		if ( handledUntil < lineStart + remainingChars ) {
+			ret += exportText( line.mid( handledUntil, remainingChars ), noAttrib, m_defaultAttribute );
+		}
+
+		if (i < (doc->lines () - 1)) ret.append ("\n");
+	}
+	ret.append ("</pre>\n");
+
+	return ret;
+}
+
+#else	// KDE < 4.4: No Highlighting Interface
+QString RKCommandHighlighter::commandToHTML (const QString r_command, HighlightingMode) {
+	return (QString ("<pre class=\"code\">") + r_command + "</pre>");
+}
+#endif
+
+/** set syntax highlighting-mode to R syntax. Outside of class, in order to allow use from the on demand code highlighter */
+void RKCommandHighlighter::setHighlighting (KTextEditor::Document *doc, HighlightingMode mode) {
+	RK_TRACE (COMMANDEDITOR);
+
+	QString mode_string = "R Script";
+	if (mode == RInteractiveSession) mode_string = "R interactive session";
+	if (!doc->setHighlightingMode (mode_string)) RK_DO (qDebug ("R syntax highlighting defintion ('%s')not found!", qPrintable (mode_string)), COMMANDEDITOR, DL_ERROR);
+}
+
+void RKCommandHighlighter::copyLinesToOutput (KTextEditor::View *view, HighlightingMode mode) {
+	RK_TRACE (COMMANDEDITOR);
+
+	// expand selection to full lines (or current line)
+	KTextEditor::Document *doc = view->document ();
+	KTextEditor::Range sel = view->selectionRange ();
+	if (!sel.isValid ()) {
+		KTextEditor::Cursor pos = view->cursorPosition ();
+		sel.setRange (KTextEditor::Cursor (pos.line (), 0),
+					  KTextEditor::Cursor (pos.line (), doc->lineLength (pos.line ())));
+	} else {
+		sel.setRange (KTextEditor::Cursor (sel.start ().line (), 0),
+					  KTextEditor::Cursor (sel.end ().line (), doc->lineLength (sel.end ().line ())));
+	}
+
+	// highlight and submit
+	QString highlighted = commandToHTML (doc->text (sel), mode);
+	if (!highlighted.isEmpty ()) {
+		RKGlobals::rInterface ()->issueCommand (".rk.cat.output (" + RObject::rQuote (highlighted) + ")\n", RCommand::App | RCommand::Silent);
+	}
+}
+
 #include "rkcommandeditorwindow.moc"
