@@ -2,7 +2,7 @@
                           rkcommandeditorwindow  -  description
                              -------------------
     begin                : Mon Aug 30 2004
-    copyright            : (C) 2004, 2006, 2007, 2009, 2010 by Thomas Friedrichsmeier
+    copyright            : (C) 2004, 2006, 2007, 2009, 2010, 2011 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -43,6 +43,7 @@ class QCloseEvent;
 class QFrame;
 class QLabel;
 class KAction;
+class QAction;
 class KActionMenu;
 class RKCommandEditorWindow;
 class KActionCollection;
@@ -64,11 +65,10 @@ public:
 	RKScriptContextProvider () {};
 	virtual ~RKScriptContextProvider () {};
 
-	/** to be implemented in subclasses. Provide some context (probably a line, but you may provide chunks in arbitrary size). If line_rev is 0, provide the line, the cursor is in. If line_rev is greater than 0, provide context before that.
+	/** to be implemented in subclasses. Provide some context, i.e. text *preceding* the cursor position (probably a line, but you may provide chunks in arbitrary size). If line_rev is 0, provide the line, the cursor is in. If line_rev is greater than 0, provide context before that.
 	@param context Place the context here
-	@param cursor_position if line_rev is 0, set this to the current column of the cursor, else set to -1
-	@returns whether context was available or not */
-	virtual bool provideContext (unsigned int line_rev, QString *context, int *cursor_position) = 0;
+	@returns a chunk of context. A null QString(), if no context was available. */
+	virtual QString provideContext (int line_rev) = 0;
 };
 
 class RObject;
@@ -98,12 +98,35 @@ private:
 	/** A timer to refresh the hint window periodically. This is a bit sorry, but it's really hard to find out, when the view has been moved, or gains/loses focus. While possible, this approach uses much less code. */
 	QTimer updater;
 	bool active;
-	QFrame *arghints_popup;
-	QLabel *arghints_popup_text;
+	QLabel *arghints_popup;
 };
 
 /** code completion model for RKCommandEditorWindow */
+#include <kdeversion.h>
+#if KDE_VERSION_MAJOR != 4
+#	error Adjust the versioning hack below!
+#endif
+// Unfortunately, MOC is not smart enough to understand the KDE_IS_VERSION macro
+#if KDE_VERSION_MINOR >= 2
+#	include <ktexteditor/codecompletionmodelcontrollerinterface.h>
+#	if KDE_VERSION_MINOR >= 5
+class RKCodeCompletionModel : public KTextEditor::CodeCompletionModel, public KTextEditor::CodeCompletionModelControllerInterface3 {
+	Q_OBJECT
+	Q_INTERFACES(KTextEditor::CodeCompletionModelControllerInterface3)
+public:
+	KTextEditor::Range completionRange (KTextEditor::View *view, const KTextEditor::Cursor &position);
+	QString filterString (KTextEditor::View *, const KTextEditor::Range &, const KTextEditor::Cursor &) { return QString (); };
+#	else
+class RKCodeCompletionModel : public KTextEditor::CodeCompletionModel, public KTextEditor::CodeCompletionModelControllerInterface {
+	Q_OBJECT
+	Q_INTERFACES(KTextEditor::CodeCompletionModelControllerInterface)
+public:
+	KTextEditor::Range completionRange (KTextEditor::View *view, const KTextEditor::Cursor &position);
+	QString filterString (KTextEditor::View *, const KTextEditor::SmartRange &, const KTextEditor::Cursor &) { return QString (); };
+#	endif
+#else
 class RKCodeCompletionModel : public KTextEditor::CodeCompletionModel {
+#endif
 public:
 	RKCodeCompletionModel (RKCommandEditorWindow* parent);
 	~RKCodeCompletionModel ();
@@ -143,8 +166,9 @@ public:
 /** open given URL. 
 @param use_r_highlighting Initialize the view to use R syntax highlighting. Use, if you're going to edit an R syntax file
 @param encoding encoding to use. If QString (), the default encoding is used.
-@param read_only Open the file in read-only mode */
-	bool openURL (const KUrl &url, const QString& encoding=QString (), bool use_r_highlighting=true, bool read_only=false);
+@param read_only Open the file in read-only mode
+@param delete_on_close File should be deleted when closing the window. Only respected with read_only=true. */
+	bool openURL (const KUrl &url, const QString& encoding=QString (), bool use_r_highlighting=true, bool read_only=false, bool delete_on_close=false);
 /** returns, whether the document was modified since the last save */
 	bool isModified ();
 /** insert the given text into the document at the current cursor position. Additionally, focuses the view */
@@ -154,8 +178,6 @@ public:
 /** copy current selection. Wrapper for use by external classes */
 	void copy ();
 
-	QString getDescription ();
-
 /** reimplemented from RKMDIWindow to return full path of file (if any) */
 	QString fullCaption ();
 
@@ -164,7 +186,7 @@ public:
 /** Return current url */
 	KUrl url ();
 
-	bool provideContext (unsigned int line_rev, QString *context, int *cursor_position);
+	QString provideContext (int line_rev);
 	QString currentCompletionWord () const;
 public slots:
 /** update Tab caption according to the current url. Display the filename-component of the URL, or - if not available - a more elaborate description of the url. Also appends a "[modified]" if appropriate */
@@ -183,6 +205,7 @@ public slots:
 	void runLine ();
 /** run the entire script */
 	void runAll ();
+	void copyLinesToOutput ();
 
 /** selection has changed. Enable / disable actions accordingly */
 	void selectionChanged (KTextEditor::View* view);
@@ -193,6 +216,9 @@ public slots:
 
 /** apply our customizations to the katepart GUI */
 	void fixupPartGUI ();
+
+	QAction* fileSaveAction () { return file_save; };
+	QAction* fileSaveAsAction () { return file_save_as; };
 protected:
 /** reimplemented from RKMDIWindow: give the editor window a chance to object to being closed (if unsaved) */
 	void closeEvent (QCloseEvent *e);
@@ -225,8 +251,6 @@ private:
 	RKCodeCompletionModel *completion_model;
 
 	QTimer *completion_timer;
-/** set syntax highlighting-mode to R syntax */
-	void setRHighlighting ();
 
 	void initializeActions (KActionCollection* ac);
 
@@ -247,6 +271,8 @@ private:
 	void addBlock (int index, const KTextEditor::Range& range);
 	void removeBlock (int index, bool was_deleted=false);
 
+	QAction *file_save, *file_save_as;
+
 	KActionMenu* actionmenu_mark_block;
 	KActionMenu* actionmenu_unmark_block;
 	KActionMenu* actionmenu_run_block;
@@ -261,6 +287,23 @@ private:
 
 	KUrl previous_autosave_url;
 	QTimer* autosave_timer;
+
+	KUrl delete_on_close;
+};
+
+/** Simple class to provide HTML highlighting for arbitrary R code. */
+class RKCommandHighlighter {
+public:
+	enum HighlightingMode {
+		RInteractiveSession,
+		RScript
+	};
+	static void copyLinesToOutput (KTextEditor::View *view, HighlightingMode mode);
+	static void setHighlighting (KTextEditor::Document *doc, HighlightingMode mode);
+	static QString commandToHTML (const QString r_command, HighlightingMode mode=RScript);
+private:
+	static KTextEditor::Document* getDoc ();
+	static KTextEditor::Document* _doc;
 };
 
 #endif
