@@ -2,7 +2,7 @@
                           rkhelpsearchwindow  -  description
                              -------------------
     begin                : Fri Feb 25 2005
-    copyright            : (C) 2005, 2006, 2007, 2009, 2010 by Thomas Friedrichsmeier
+    copyright            : (C) 2005, 2006, 2007, 2009, 2010, 2011 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -38,6 +38,7 @@
 #include "../debug.h"
 #include "../rkglobals.h"
 #include "../rkward.h"
+#include "../core/robject.h"
 #include "../misc/rkcommonfunctions.h"
 #include "../misc/rkdummypart.h"
 
@@ -161,7 +162,7 @@ void RKHelpSearchWindow::getFunctionHelp (const QString &function_name, const QS
 	RK_TRACE (APP);
 
 	// we use .rk.getHelp() instead of plain help() to receive an error, if no help could be found
-	QString command = ".rk.getHelp(\"" + function_name + '\"';
+	QString command = ".rk.getHelp(" + RObject::rQuote (function_name);
 	if (!package.isEmpty ()) command.append (", package=" + package);
 	command.append (")");
 
@@ -196,7 +197,7 @@ void RKHelpSearchWindow::slotFindButtonClicked () {
 
 	QString fields = fieldsList->itemData (fieldsList->currentIndex ()).toString ();
 
-	QString s = ".rk.get.search.results (\"" + field->currentText () + "\",agrep=" + agrep + ", ignore.case=" + ignoreCase + package + ", fields=" + fields +")";
+	QString s = ".rk.get.search.results (" + RObject::rQuote (field->currentText ()) + ", agrep=" + agrep + ", ignore.case=" + ignoreCase + package + ", fields=" + fields +")";
 	
 	RKGlobals::rInterface ()->issueCommand (s, RCommand::App | RCommand::Sync | RCommand::GetStringVector, QString::null, this, HELP_SEARCH, 0);
 	setEnabled (false);
@@ -219,11 +220,14 @@ void RKHelpSearchWindow::resultDoubleClicked (const QModelIndex& index) {
 void RKHelpSearchWindow::rCommandDone (RCommand *command) {
 	RK_TRACE (APP);
 	if (command->getFlags () == HELP_SEARCH) {
+		if (command->failed ()) {
+			RK_ASSERT (false);
+			return;
+		}
 		RK_ASSERT ((command->getDataLength () % 3) == 0);
 		RK_ASSERT (command->getDataType () == RData::StringVector);
 
-		results->setResults (command->getStringVector (), command->getDataLength () / 3);
-		command->detachData ();		// now owned by the model
+		results->setResults (command->getStringVector ());
 
 		for (int i = 0; i < COL_COUNT; ++i) results_view->resizeColumnToContents (i);
 		setEnabled(true);
@@ -247,22 +251,18 @@ void RKHelpSearchWindow::rCommandDone (RCommand *command) {
 RKHelpSearchResultsModel::RKHelpSearchResultsModel (QObject *parent) : QAbstractTableModel (parent) {
 	RK_TRACE (APP);
 
-	results = 0;
 	result_count = 0;
 }
 
 RKHelpSearchResultsModel::~RKHelpSearchResultsModel () {
 	RK_TRACE (APP);
-
-	delete [] results;
 }
 
-void RKHelpSearchResultsModel::setResults (QString* new_results, int new_result_count) {
+void RKHelpSearchResultsModel::setResults (const QStringList &new_results) {
 	RK_TRACE (APP);
 
-	delete [] results;
 	results = new_results;
-	result_count = new_result_count;
+	result_count = results.size () / 3;
 
 	reset ();
 }
@@ -285,7 +285,7 @@ QVariant RKHelpSearchResultsModel::data (const QModelIndex& index, int role) con
 	// easier typing
 	int row = index.row ();
 	int col = index.column ();
-	if (results) {
+	if (result_count) {
 		if (role == Qt::DisplayRole || role == Qt::ToolTipRole) {
 			if (row < result_count) {
 				if (col < COL_COUNT) {
