@@ -97,16 +97,10 @@ void RKHTMLWindow::fixupPartGUI () {
 	RKCommonFunctions::removeContainers (khtmlpart, QString ("tools,security,extraToolBar,saveBackground,saveFrame,printFrame,kget_menu").split (','), true);
 }
 
-QString RKHTMLWindow::getDescription () {
+KUrl RKHTMLWindow::restorableUrl () {
 	RK_TRACE (APP);
 
-	if (window_mode == HTMLOutputWindow) {
-		return ("output:" + current_url.url ());
-
-	} else {
-		QString fixed_url = current_url.url ().replace (RKSettingsModuleR::helpBaseUrl(), "rkward://RHELPBASE");
-		return ("help:" + fixed_url);
-	}
+	return (current_url.url ().replace (RKSettingsModuleR::helpBaseUrl(), "rkward://RHELPBASE"));
 }
 
 bool RKHTMLWindow::isModified () {
@@ -361,7 +355,7 @@ void RKHTMLWindow::useMode (WindowMode new_mode) {
 	if (window_mode == new_mode) return;
 
 	if (new_mode == HTMLOutputWindow) {
-		type = RKMDIWindow::OutputWindow;
+		type = RKMDIWindow::OutputWindow | RKMDIWindow::DocumentWindow;
 		setWindowIcon (RKStandardIcons::getIcon (RKStandardIcons::WindowOutput));
 
 		print->setText (i18n ("Print output"));
@@ -375,7 +369,7 @@ void RKHTMLWindow::useMode (WindowMode new_mode) {
 	} else {
 		RK_ASSERT (new_mode == HTMLHelpWindow);
 
-		type = RKMDIWindow::HelpWindow;
+		type = RKMDIWindow::HelpWindow | RKMDIWindow::DocumentWindow;
 		setWindowIcon (RKStandardIcons::getIcon (RKStandardIcons::WindowHelp));
 
 		print->setText (i18n ("Print page"));
