@@ -2,7 +2,7 @@
                           rkmdiwindow  -  description
                              -------------------
     begin                : Tue Sep 26 2006
-    copyright            : (C) 2006, 2007, 2008, 2009, 2010 by Thomas Friedrichsmeier
+    copyright            : (C) 2006, 2007, 2008, 2009, 2010, 2011 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -32,6 +32,7 @@
 #include "rkworkplace.h"
 #include "rkworkplaceview.h"
 #include "rktoolwindowbar.h"
+#include "rktoolwindowlist.h"
 #include "../settings/rksettingsmodulegeneral.h"
 #include "../misc/rkstandardicons.h"
 #include "../misc/rkxmlguisyncer.h"
@@ -72,6 +73,7 @@ RKMDIWindow::RKMDIWindow (QWidget *parent, int type, bool tool_window, const cha
 RKMDIWindow::~RKMDIWindow () {
 	RK_TRACE (APP);
 
+	if (isToolWindow ()) RKToolWindowList::unregisterToolWindow (this);
 	delete standard_client;
 }
 
@@ -116,8 +118,12 @@ void RKMDIWindow::activate (bool with_focus) {
 	QWidget *old_focus = qApp->focusWidget ();
 
 	if (isToolWindow ()) {
-		RK_ASSERT (tool_window_bar);
-		tool_window_bar->showWidget (this);
+		if (tool_window_bar) tool_window_bar->showWidget (this);
+		else if (!isVisible ()) RKWorkplace::mainWorkplace ()->detachWindow (this, true);
+		else {
+			topLevelWidget ()->show ();
+			topLevelWidget ()->raise ();
+		}
 	} else {
 		if (isAttached ()) RKWorkplace::mainWorkplace ()->view ()->setActivePage (this);
 		else {
@@ -126,6 +132,7 @@ void RKMDIWindow::activate (bool with_focus) {
 		}
 	}
 
+	emit (windowActivated (this));
 	if (with_focus) {
 		if (old_focus) old_focus->clearFocus ();
 		topLevelWidget ()->activateWindow ();
@@ -148,8 +155,8 @@ bool RKMDIWindow::close (bool also_delete) {
 			RKWorkplace::mainWorkplace ()->attachWindow (this);
 		}
 
-		RK_ASSERT (tool_window_bar);
-		tool_window_bar->hideWidget (this);
+		if (tool_window_bar) tool_window_bar->hideWidget (this);
+		else hide ();
 		return true;
 	}
 
@@ -164,7 +171,7 @@ bool RKMDIWindow::close (bool also_delete) {
 			}
 			// WORKAROUND end
 
-			deleteLater ();
+			delete this;	// Note: using deleteLater(), here does not work well while restoring workplaces (window is not fully removed from workplace before restoring)
 		}
 		return closed;
 	} else {
@@ -181,8 +188,7 @@ void RKMDIWindow::prepareToBeDetached () {
 	RK_TRACE (APP);
 
 	if (isToolWindow ()) {
-		RK_ASSERT (tool_window_bar);
-		tool_window_bar->hideWidget (this);
+		if (tool_window_bar) tool_window_bar->hideWidget (this);
 	}
 }
 
