@@ -2,7 +2,7 @@
                           rkworkplace  -  description
                              -------------------
     begin                : Thu Sep 21 2006
-    copyright            : (C) 2006, 2007, 2009, 2010 by Thomas Friedrichsmeier
+    copyright            : (C) 2006, 2007, 2009, 2010, 2011 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -28,11 +28,14 @@
 #include <krun.h>
 #include <kmimetype.h>
 
+#include <QFileInfo>
+
 #include "detachedwindowcontainer.h"
 #include "rkcommandeditorwindow.h"
 #include "rkhtmlwindow.h"
 #include "rkworkplaceview.h"
 #include "rktoolwindowbar.h"
+#include "rktoolwindowlist.h"
 #include "../core/robject.h"
 #include "../core/rcontainerobject.h"
 #include "../core/robjectlist.h"
@@ -49,8 +52,6 @@
 
 #include "../debug.h"
 
-#define RESTORE_WORKPLACE_COMMAND 1
-
 // static
 RKWorkplace *RKWorkplace::main_workplace = 0;
 
@@ -63,35 +64,32 @@ RKWorkplace::RKWorkplace (QWidget *parent) : QWidget (parent) {
 	/* Splitter setup contains heavy copying from Kate's katemdi! */
 	KVBox *vbox = new KVBox (this);
 
-	tool_window_bars[KMultiTabBar::Top] = new RKToolWindowBar (KMultiTabBar::Top, vbox);
+	tool_window_bars[RKToolWindowList::Top] = new RKToolWindowBar (KMultiTabBar::Top, vbox);
 	vert_splitter = new QSplitter (Qt::Vertical, vbox);
 	vert_splitter->setOpaqueResize (KGlobalSettings::opaqueResize ());
-	tool_window_bars[KMultiTabBar::Top]->setSplitter (vert_splitter);
+	tool_window_bars[RKToolWindowList::Top]->setSplitter (vert_splitter);
 
 	KHBox *hbox = new KHBox (vert_splitter);
 	vert_splitter->setCollapsible (vert_splitter->indexOf (hbox), false);
 	vert_splitter->setStretchFactor (vert_splitter->indexOf (hbox), 1);
 
-	tool_window_bars[KMultiTabBar::Left] = new RKToolWindowBar (KMultiTabBar::Left, hbox);
+	tool_window_bars[RKToolWindowList::Left] = new RKToolWindowBar (KMultiTabBar::Left, hbox);
 	horiz_splitter = new QSplitter (Qt::Horizontal, hbox);
 	horiz_splitter->setOpaqueResize (KGlobalSettings::opaqueResize ());
-	tool_window_bars[KMultiTabBar::Left]->setSplitter (horiz_splitter);
+	tool_window_bars[RKToolWindowList::Left]->setSplitter (horiz_splitter);
 
 	wview = new RKWorkplaceView (horiz_splitter);
 	horiz_splitter->setCollapsible (horiz_splitter->indexOf (wview), false);
 	horiz_splitter->setStretchFactor(horiz_splitter->indexOf (wview), 1);
 
-	tool_window_bars[KMultiTabBar::Bottom] = new RKToolWindowBar (KMultiTabBar::Bottom, vbox);
-	tool_window_bars[KMultiTabBar::Bottom]->setSplitter (vert_splitter);
+	tool_window_bars[RKToolWindowList::Bottom] = new RKToolWindowBar (KMultiTabBar::Bottom, vbox);
+	tool_window_bars[RKToolWindowList::Bottom]->setSplitter (vert_splitter);
 
-	tool_window_bars[KMultiTabBar::Right] = new RKToolWindowBar (KMultiTabBar::Right, hbox);
-	tool_window_bars[KMultiTabBar::Right]->setSplitter (horiz_splitter);
+	tool_window_bars[RKToolWindowList::Right] = new RKToolWindowBar (KMultiTabBar::Right, hbox);
+	tool_window_bars[RKToolWindowList::Right]->setSplitter (horiz_splitter);
 
 	KConfigGroup toolbar_config = KGlobal::config ()->group ("ToolwindowBars");
-	tool_window_bars[KMultiTabBar::Top]->restoreSize (toolbar_config);
-	tool_window_bars[KMultiTabBar::Left]->restoreSize (toolbar_config);
-	tool_window_bars[KMultiTabBar::Bottom]->restoreSize (toolbar_config);
-	tool_window_bars[KMultiTabBar::Right]->restoreSize (toolbar_config);
+	for (int i = 0; i < TOOL_WINDOW_BAR_COUNT; ++i) tool_window_bars[i]->restoreSize (toolbar_config);
 
 	// now add it all to this widget
 	QVBoxLayout *box = new QVBoxLayout (this);
@@ -113,16 +111,12 @@ void RKWorkplace::saveSettings () {
 	RK_TRACE (APP);
 
 	KConfigGroup toolbar_config = KGlobal::config ()->group ("ToolwindowBars");
-	tool_window_bars[KMultiTabBar::Top]->saveSize (toolbar_config);
-	tool_window_bars[KMultiTabBar::Left]->saveSize (toolbar_config);
-	tool_window_bars[KMultiTabBar::Bottom]->saveSize (toolbar_config);
-	tool_window_bars[KMultiTabBar::Right]->saveSize (toolbar_config);
+	for (int i = 0; i < TOOL_WINDOW_BAR_COUNT; ++i) tool_window_bars[i]->saveSize (toolbar_config);
 }
 
-void RKWorkplace::initActions (KActionCollection *ac, const char *prev_id, const char *next_id, const char *left_id, const char *right_id) {
+void RKWorkplace::initActions (KActionCollection *ac, const char *left_id, const char *right_id) {
 	RK_TRACE (APP);
 
-	history->initActions (ac, prev_id, next_id);
 	wview->initActions (ac, left_id, right_id);
 }
 
@@ -141,7 +135,8 @@ void RKWorkplace::attachWindow (RKMDIWindow *window) {
 	// all the rest is done, even if the window was previously "Attached", as this may also mean it was freshly created
 	window->state = RKMDIWindow::Attached;
 	if (window->isToolWindow ()) {
-		window->tool_window_bar->reclaimDetached (window);
+		if (!window->tool_window_bar) placeInToolWindowBar (window, RKToolWindowList::Bottom);
+		else window->tool_window_bar->reclaimDetached (window);
 	} else {
 		view ()->addWindow (window);
 		view ()->topLevelWidget ()->raise ();
@@ -168,26 +163,46 @@ void RKWorkplace::detachWindow (RKMDIWindow *window, bool was_attached) {
 
 	DetachedWindowContainer *detached = new DetachedWindowContainer (window);
 	detached->show ();
+	if (!was_attached) window->activate ();
 }
 
 void RKWorkplace::addWindow (RKMDIWindow *window, bool attached) {
 	RK_TRACE (APP);
 
 	windows.append (window);
-	connect (window, SIGNAL (destroyed (QObject *)), this, SLOT (windowDestroyed (QObject *)));
+	connect (window, SIGNAL (destroyed (QObject *)), this, SLOT (removeWindow (QObject *)));
 	connect (window, SIGNAL (windowActivated(RKMDIWindow*)), history, SLOT (windowActivated(RKMDIWindow*)));
+	if (window->isToolWindow () && !window->tool_window_bar) return;
 	if (attached) attachWindow (window);
 	else detachWindow (window, false);
 }
 
-void RKWorkplace::placeInToolWindowBar (RKMDIWindow *window, KMultiTabBar::KMultiTabBarPosition position) {
+void RKWorkplace::placeToolWindows() {
+	RK_TRACE (APP);
+
+	foreach (const RKToolWindowList::ToolWindowRepresentation rep, RKToolWindowList::registeredToolWindows ()) {
+		placeInToolWindowBar (rep.window, rep.default_placement);
+		getHistory ()->popLastWindow (rep.window);	// windows send a spurious activation signal triggered from KPartsManager::addPart(), so we pop them, again
+	}
+}
+
+void RKWorkplace::placeInToolWindowBar (RKMDIWindow *window, int position) {
 	RK_TRACE (APP);
 
 	RK_ASSERT (window->isToolWindow ());
-	tool_window_bars[position]->addWidget (window);
-	if (!windows.contains (window)) {	// must be new
+	if ((position < 0) || (position >= TOOL_WINDOW_BAR_COUNT)) {
+		RK_ASSERT (position == RKToolWindowList::Nowhere);	// should never happen...
+		position = RKToolWindowList::Nowhere;		// ... but let's set this explicitly, in case of a broken workplace representation
+	}
+	if (position == RKToolWindowList::Nowhere) {
+		if (window->tool_window_bar) window->tool_window_bar->removeWidget (window);
+	} else {
+		tool_window_bars[position]->addWidget (window);
+	}
+
+	if (!windows.contains (window)) {	// first time, we see this window?
 		addWindow (window, true);
-		RKWardMainWindow::getMain ()->partManager ()->addPart (window->getPart ());
+		if (window->isAttached () && window->tool_window_bar) RKWardMainWindow::getMain ()->partManager ()->addPart (window->getPart ());
 	}
 }
 
@@ -216,7 +231,7 @@ bool RKWorkplace::openAnyUrl (const KUrl &url) {
 	return false;
 }
 
-bool RKWorkplace::openScriptEditor (const KUrl &url, const QString& encoding, bool use_r_highlighting, bool read_only, const QString &force_caption) {
+RKMDIWindow* RKWorkplace::openScriptEditor (const KUrl &url, const QString& encoding, bool use_r_highlighting, bool read_only, const QString &force_caption, bool delete_on_close) {
 	RK_TRACE (APP);
 
 // is this url already opened?
@@ -226,7 +241,7 @@ bool RKWorkplace::openScriptEditor (const KUrl &url, const QString& encoding, bo
 			  KUrl ourl = static_cast<RKCommandEditorWindow *> (*it)->url ();
 			  if (url == ourl) {
 				  (*it)->activate ();
-				  return true;
+				  return (*it);
 			  }
 		}
 	}
@@ -234,24 +249,24 @@ bool RKWorkplace::openScriptEditor (const KUrl &url, const QString& encoding, bo
 	RKCommandEditorWindow *editor = new RKCommandEditorWindow (view (), use_r_highlighting);
 
 	if (!url.isEmpty ()) {
-		if (!editor->openURL (url, encoding, use_r_highlighting, read_only)) {
+		if (!editor->openURL (url, encoding, use_r_highlighting, read_only, delete_on_close)) {
 			delete editor;
 			KMessageBox::messageBox (view (), KMessageBox::Error, i18n ("Unable to open \"%1\"", url.prettyUrl ()), i18n ("Could not open command file"));
-			return false;
+			return 0;
 		}
 	}
 
 	if (!force_caption.isEmpty ()) editor->setCaption (force_caption);
 	addWindow (editor);
-	return true;
+	return (editor);
 }
 
-void RKWorkplace::openHelpWindow (const KUrl &url, bool only_once) {
+RKMDIWindow* RKWorkplace::openHelpWindow (const KUrl &url, bool only_once) {
 	RK_TRACE (APP);
 
 	if (url.isEmpty ()) {
 		RK_ASSERT (false);
-		return;
+		return 0;
 	}
 
 	if (only_once) {
@@ -259,7 +274,7 @@ void RKWorkplace::openHelpWindow (const KUrl &url, bool only_once) {
 		for (RKWorkplaceObjectList::const_iterator it = help_windows.constBegin (); it != help_windows.constEnd (); ++it) {
 			if (static_cast<RKHTMLWindow *> (*it)->url ().equals (url, KUrl::CompareWithoutTrailingSlash | KUrl::CompareWithoutFragment)) {
 				(*it)->activate ();
-				return;
+				return (*it);
 			}
 		}
 	}
@@ -267,9 +282,10 @@ void RKWorkplace::openHelpWindow (const KUrl &url, bool only_once) {
 	RKHTMLWindow *hw = new RKHTMLWindow (view (), RKHTMLWindow::HTMLHelpWindow);
 	hw->openURL (url);
 	addWindow (hw);
+	return (hw);
 }
 
-void RKWorkplace::openOutputWindow (const KUrl &url) {
+RKMDIWindow* RKWorkplace::openOutputWindow (const KUrl &url) {
 	RK_TRACE (APP);
 
 	RKHTMLWindow *w = RKOutputWindowManager::self ()->getCurrentOutputWindow ();
@@ -278,6 +294,7 @@ void RKWorkplace::openOutputWindow (const KUrl &url) {
 	} else {
 		w->activate ();
 	}
+	return (w);
 }
 
 void RKWorkplace::newX11Window (WId window_to_embed, int device_number) {
@@ -309,7 +326,7 @@ bool RKWorkplace::canEditObject (RObject *object) {
 	
 	if (object->isDataFrame ()) {
 		return true;
-	} else if (object->isVariable () && object->getContainer ()->isDataFrame ()) {
+	} else if (object->isVariable () && object->parentObject ()->isDataFrame ()) {
 		return true;
 	}
 	return false;
@@ -327,22 +344,23 @@ RKEditor* RKWorkplace::editNewDataFrame (const QString &name) {
 
 RKEditor *RKWorkplace::editObject (RObject *object) {
 	RK_TRACE (APP);
+	RK_ASSERT (object);
 
 	RObject *iobj = object;
 	RKEditor *ed = 0;
-	RKEditor *existing_editor = RKGlobals::tracker ()->objectEditor (object);
+	RKEditor *existing_editor = object->editor ();
 	if (!existing_editor) {
 		if (!iobj->isDataFrame ()) {
-			if (iobj->isVariable () && iobj->getContainer ()->isDataFrame ()) {
-				iobj = iobj->getContainer ();
+			if (iobj->isVariable () && iobj->parentObject ()->isDataFrame ()) {
+				iobj = iobj->parentObject ();
 			} else {
 				return 0;
 			}
 		}
 
 		unsigned long size = 1;
-		for (unsigned int i = 0; i < iobj->numDimensions (); ++i) {
-			size *= iobj->getDimension (i);
+		foreach (int dim, iobj->getDimensions ()) {
+			size *= dim;
 		}
 		if ((RKSettingsModuleGeneral::warnLargeObjectThreshold () != 0) && (size > RKSettingsModuleGeneral::warnLargeObjectThreshold ())) {
 			if (KMessageBox::warningContinueCancel (view (), i18n ("You are about to edit object \"%1\", which is very large (%2 fields). RKWard is not optimized to handle very large objects in the built in data editor. This will use a lot of memory, and - depending on your system - might be very slow. For large objects it is generally recommended to edit using command line means or to split into smaller chunks before editing. On the other hand, if you have enough memory, or the data is simple enough (numeric data is easier to handle, than factor), editing may not be a problem at all. You can configure this warning (or turn it off entirely) under Settings->Configure RKWard->General.\nReally edit object?", iobj->getFullName (), size), i18n ("About to edit very large object")) != KMessageBox::Continue) {
@@ -375,9 +393,9 @@ void RKWorkplace::closeWindow (RKMDIWindow *window) {
 	RK_ASSERT (windows.contains (window));
 
 	bool tool_window = window->isToolWindow ();
-	window->close (true);		// all the rest should happen in windowDestroyed ()
+	window->close (true);		// all the rest should happen in removeWindow ()
 	
-	if (tool_window) windowRemoved ();	// for regular windows, this happens in windowDestroyed(), already
+	if (tool_window) windowRemoved ();	// for regular windows, this happens in removeWindow(), already
 }
 
 void RKWorkplace::closeActiveWindow () {
@@ -402,18 +420,20 @@ RKWorkplace::RKWorkplaceObjectList RKWorkplace::getObjectList (int type, int sta
 void RKWorkplace::closeAll (int type, int state) {
 	RK_TRACE (APP);
 
+	RKWardMainWindow::getMain ()->lockGUIRebuild (true);
 	RKWorkplaceObjectList list_to_close = getObjectList (type, state);
 	for (RKWorkplaceObjectList::const_iterator it = list_to_close.constBegin (); it != list_to_close.constEnd (); ++it) {
 		closeWindow (*it);
 	}
+	RKWardMainWindow::getMain ()->lockGUIRebuild (false);
 }
 
-void RKWorkplace::windowDestroyed (QObject *object) {
+void RKWorkplace::removeWindow (QObject *object) {
 	RK_TRACE (APP);
 	RKMDIWindow *window = static_cast<RKMDIWindow *> (object);
 
 	// remove from history first (otherwise, we might find it there, when trying to activate a new window)
-	history->windowDestroyed (window);
+	history->removeWindow (window);
 
 	// WARNING: the window is dead. Don't call any functions on it.
 
@@ -437,11 +457,9 @@ void RKWorkplace::windowRemoved () {
 	}
 
 	// some document window in the history? Try that.
-	if (history->haveNext ()) {
-		history->next ();
-		return;
-	} else if (history->havePrev ()) {
-		history->prev ();
+	window = history->previousDocumentWindow ();
+	if (window) {
+		window->activate (true);
 		return;
 	}
 
@@ -482,23 +500,56 @@ RKMDIWindow *RKWorkplace::activeWindow (RKMDIWindow::State state) {
 	return (ret);
 }
 
-QString RKWorkplace::makeWorkplaceDescription (const QString &sep, bool quote) {
+QStringList RKWorkplace::makeWorkplaceDescription () {
 	RK_TRACE (APP);
 
+	QStringList workplace_description;
+
+	// first, save the base directory of the workplace. This allows us to cope better with moved workspaces while restoring.
+	KUrl base_url = RObjectList::getObjectList ()->getWorkspaceURL ();
+	base_url.setPath (base_url.directory ());
+	if (base_url.isLocalFile () && base_url.hasPath ()) workplace_description.append ("base::::" + base_url.url ());
+
 	// window order in the workplace view may have changed with respect to our list. Thus we first generate a properly sorted list
 	RKWorkplaceObjectList list = getObjectList (RKMDIWindow::DocumentWindow, RKMDIWindow::Detached);
 	for (int i=0; i < wview->count (); ++i) {
 		list.append (static_cast<RKMDIWindow*> (wview->widget (i)));
 	}
-	
-	QString workplace_description;
-	bool first = true;
+	list.append (getObjectList (RKMDIWindow::ToolWindow, RKMDIWindow::AnyWindowState));
 	foreach (RKMDIWindow *win, list) {
-		if (first) first = false;
-		else workplace_description.append (sep);
-
-		if (!quote) workplace_description.append (win->getDescription ());
-		else workplace_description.append (RObject::rQuote (win->getDescription ()));
+		QString type, specification;
+		QStringList params;
+		if (win->isType (RKMDIWindow::DataEditorWindow)) {
+			type = "data";
+			specification = static_cast<RKEditor*> (win)->getObject ()->getFullName ();
+		} else if (win->isType (RKMDIWindow::CommandEditorWindow)) {
+			type = "script";
+			specification = static_cast<RKCommandEditorWindow*> (win)->url ().url ();
+		} else if (win->isType (RKMDIWindow::OutputWindow)) {
+			type = "output";
+			specification = static_cast<RKHTMLWindow*> (win)->url ().url ();
+		} else if (win->isType (RKMDIWindow::HelpWindow)) {
+			type = "help";
+			specification = static_cast<RKHTMLWindow*> (win)->restorableUrl ().url ();
+		} else if (win->isToolWindow ()) {
+			type = RKToolWindowList::idOfWindow (win);
+		}
+		if (!type.isEmpty ()) {
+			if (!win->isAttached ()) {
+				params.append (QString ("detached,") + QString::number (win->x ()) + ',' + QString::number (win->y ()) + ',' + QString::number (win->width ()) + ',' + QString::number (win->height ()));
+			}
+			if (win->isToolWindow ()) {
+				int sidebar = RKToolWindowList::Nowhere;
+				for (int i = 0; i < TOOL_WINDOW_BAR_COUNT; ++i) {
+					if (win->tool_window_bar == tool_window_bars[i]) {
+						sidebar = i;
+						break;
+					}
+				}
+				params.append (QString ("sidebar,") + QString::number (sidebar));
+			}
+			workplace_description.append (type + "::" + params.join (":") + "::" + specification);
+		}
 	}
 	return workplace_description;
 }
@@ -507,171 +558,294 @@ void RKWorkplace::saveWorkplace (RCommandChain *chain) {
 	RK_TRACE (APP);
 	if (RKSettingsModuleGeneral::workplaceSaveMode () != RKSettingsModuleGeneral::SaveWorkplaceWithWorkspace) return;
 
-	QString workplace_description = ".rk.workplace.save <- c (" + makeWorkplaceDescription (", ", true) + ')';
-
-	RKGlobals::rInterface ()->issueCommand (workplace_description, RCommand::App | RCommand::Sync, i18n ("Save Workplace layout"), 0, 0, chain); 
+	RKGlobals::rInterface ()->issueCommand ("rk.save.workplace(description=" + RObject::rQuote (makeWorkplaceDescription().join ("\n")) + ")", RCommand::App, i18n ("Save Workplace layout"), 0, 0, chain);
 }
 
 void RKWorkplace::restoreWorkplace (RCommandChain *chain) {
 	RK_TRACE (APP);
 	if (RKSettingsModuleGeneral::workplaceSaveMode () != RKSettingsModuleGeneral::SaveWorkplaceWithWorkspace) return;
 
-	RKGlobals::rInterface ()->issueCommand (".rk.workplace.save", RCommand::App | RCommand::Sync | RCommand::GetStringVector, i18n ("Restore Workplace layout"), this, RESTORE_WORKPLACE_COMMAND, chain);
+	RKGlobals::rInterface ()->issueCommand ("rk.restore.workplace()", RCommand::App, i18n ("Restore Workplace layout"), 0, 0, chain);
 }
 
-void RKWorkplace::restoreWorkplace (const QString &description) {
+KUrl checkAdjustRestoredUrl (const QString &_url, const QString old_base) {
+	KUrl url (_url);
+
+	if (old_base.isEmpty ()) return (url);
+	KUrl new_base_url = RObjectList::getObjectList ()->getWorkspaceURL ();
+	new_base_url.setPath (new_base_url.directory ());
+	if (new_base_url.isEmpty ()) return (url);
+	KUrl old_base_url (old_base);
+	if (old_base_url == new_base_url) return (url);
+
+	// TODO: Should we also care about non-local files? In theory: yes, but stat'ing remote files for existence can take a long time.
+	if (!(old_base_url.isLocalFile () && new_base_url.isLocalFile () && url.isLocalFile ())) return (url);
+
+	// if the file exists, unadjusted, return it.
+	if (QFileInfo (url.toLocalFile ()).exists ()) return (url);
+
+	// check whether a file exists for the adjusted url
+	KUrl relative = KUrl::fromLocalFile (new_base_url.path () + '/' + KUrl::relativePath (old_base_url.path (), url.path ()));
+	relative.cleanPath ();
+	if (QFileInfo (relative.toLocalFile ()).exists ()) return (relative);
+	return (url);
+}
+
+void RKWorkplace::restoreWorkplace (const QStringList &description) {
 	RK_TRACE (APP);
 
-	QStringList list = description.split ("\n");
-	for (QStringList::const_iterator it = list.constBegin (); it != list.constEnd (); ++it) {
-		restoreWorkplaceItem (*it);
+	RKWardMainWindow::getMain ()->lockGUIRebuild (true);
+	QString base;
+	for (int i = 0; i < description.size (); ++i) {
+		// Item format for rkward <= 0.5.4: "type:specification"
+		// Item format for rkward <= 0.5.5: "type::[optional_params1[:optional_params2[:...]]]::specification"
+		int typeend = description[i].indexOf (':');
+		if ((typeend < 0) || (typeend >= (description[i].size () - 1))) {
+			RK_ASSERT (false);
+			continue;
+		}
+		QString type, specification;
+		QStringList params;
+		type = description[i].left (typeend);
+		if (description[i].at (typeend + 1) == ':') {	// rkward 0.5.5 or later
+			int specstart = description[i].indexOf ("::", typeend + 2);
+			if (specstart < typeend) {
+				RK_ASSERT (false);
+				continue;
+			}
+			params = description[i].mid (typeend + 2, specstart - typeend - 2).split (':', QString::SkipEmptyParts);
+			specification = description[i].mid (specstart + 2);
+		} else {
+			specification = description[i].mid (typeend + 1);
+		}
+
+		RKMDIWindow *win = 0;
+		if (type == "base") {
+			RK_ASSERT (i == 0);
+			base = specification;
+		} else if (type == "data") {
+			RObject *object = RObjectList::getObjectList ()->findObject (specification);
+			if (object) win = editObject (object);
+		} else if (type == "script") {
+			win = openScriptEditor (checkAdjustRestoredUrl (specification, base));
+		} else if (type == "output") {
+			win = openOutputWindow (checkAdjustRestoredUrl (specification, base));
+		} else if (type == "help") {
+			win = openHelpWindow (checkAdjustRestoredUrl (specification, base), true);
+		} else {
+			win = RKToolWindowList::findToolWindowById (type);
+			RK_ASSERT (win);
+		}
+
+		// apply generic window parameters
+		if (win) {
+			for (int p = 0; p < params.size (); ++p) {
+				if (params[p].startsWith ("sidebar")) {
+					int position = params[p].section (',', 1).toInt ();
+					placeInToolWindowBar (win, position);
+				}
+				if (params[p].startsWith ("detached")) {
+					QStringList geom = params[p].split (',');
+					win->hide ();
+					win->setGeometry (geom.value (1).toInt (), geom.value (2).toInt (), geom.value (3).toInt (), geom.value (4).toInt ());
+					detachWindow (win);
+				}
+			}
+		}
 	}
+	RKWardMainWindow::getMain ()->lockGUIRebuild (false);
 }
 
-void RKWorkplace::clearWorkplaceDescription (RCommandChain *chain) {
-	RK_TRACE (APP);
-	if (RKSettingsModuleGeneral::workplaceSaveMode () != RKSettingsModuleGeneral::SaveWorkplaceWithWorkspace) return;
-
-	RKGlobals::rInterface ()->issueCommand ("remove (.rk.workplace.save)", RCommand::App | RCommand::Sync | RCommand::ObjectListUpdate, QString::null, 0, 0, chain); 
-}
-
-void RKWorkplace::rCommandDone (RCommand *command) {
-	RK_TRACE (APP);
-
-	RK_ASSERT (command->getFlags () == RESTORE_WORKPLACE_COMMAND);
-	for (unsigned int i = 0; i < command->getDataLength (); ++i) {
-		restoreWorkplaceItem (command->getStringVector ()[i]);
-	}
-}
-
-void RKWorkplace::restoreWorkplaceItem (const QString &desc) {
-	RK_TRACE (APP);
-
-	QString type = desc.section (QChar (':'), 0, 0);
-	QString specification = desc.section (QChar (':'), 1);
-
-	if (type == "data") {
-		RObject *object = RObjectList::getObjectList ()->findObject (specification);
-		if (object) editObject (object);
-	} else if (type == "script") {
-		openScriptEditor (specification);
-	} else if (type == "output") {
-		openOutputWindow (specification);
-	} else if (type == "help") {
-		openHelpWindow (specification, true);
-	} else {
-		RK_ASSERT (false);
-	}
-}
-
-
 ///////////////////////// END RKWorkplace ////////////////////////////
 ///////////////////// BEGIN RKMDIWindowHistory ///////////////////////
 
+#include "../misc/rkstandardicons.h"
+#include <QListWidget>
+
+class RKMDIWindowHistoryWidget : public QListWidget {
+public:
+	RKMDIWindowHistoryWidget () : QListWidget (0) {
+		RK_TRACE (APP);
+
+		current = 0;
+		setFocusPolicy (Qt::StrongFocus);
+		setWindowFlags (Qt::Popup);
+	}
+
+	~RKMDIWindowHistoryWidget () {
+		RK_TRACE (APP);
+	}
+
+	void update (const QList<RKMDIWindow*> windows) {
+		RK_TRACE (APP);
+
+		clear ();
+		_windows = windows;
+		for (int i = windows.count () - 1; i >= 0; --i) {		// most recent top
+			RKMDIWindow *win = windows[i];
+			QListWidgetItem *item = new QListWidgetItem (this);
+			item->setIcon (RKStandardIcons::iconForWindow (win));
+			item->setText (win->windowTitle ());
+		}
+		if (current >= count ()) current = count () - 1;
+		if (current < 0) {
+			hide ();
+			return;
+		}
+		setCurrentRow (current);
+	}
+
+	void next () {
+		RK_TRACE (APP);
+
+		if (--current < 0) current = count () - 1;
+		setCurrentRow (current);
+	}
+
+	void prev () {
+		RK_TRACE (APP);
+
+		if (++current >= count ()) current = 0;
+		setCurrentRow (current);
+	}
+
+private:
+	void focusOutEvent (QFocusEvent *) {
+		RK_TRACE (APP);
+
+		deleteLater ();
+	}
+
+	void keyReleaseEvent (QKeyEvent *ev) {
+		RK_TRACE (APP);
+
+		if (ev->modifiers () == Qt::NoModifier) {
+			commit ();
+		}
+	}
+
+	void mouseReleaseEvent (QMouseEvent *ev) {
+		RK_TRACE (APP);
+
+		// HACK to get by without slots, and the associated moc'ing
+		QListWidget::mouseReleaseEvent (ev);
+		commit ();
+	}
+
+	void commit () {
+		RK_TRACE (APP);
+
+		current = currentRow ();
+		if ((current > 0) && (current < count ())) {
+			RKMDIWindow *win = _windows.value (count () - 1 - current);
+			RK_ASSERT (win);
+			win->activate (true);
+		}
+		deleteLater ();
+	}
+
+	int current;
+	QList<RKMDIWindow*> _windows;
+};
+
 RKMDIWindowHistory::RKMDIWindowHistory (QObject *parent) : QObject (parent) {
 	RK_TRACE (APP);
 
-	current = 0;
-	prev_action = next_action = 0;
+	switcher = 0;
 }
 
 RKMDIWindowHistory::~RKMDIWindowHistory () {
 	RK_TRACE (APP);
 
-	RK_DO (qDebug ("Remaining windows in history: forward: %d, backward: %d", forward_list.count (), back_list.count ()), APP, DL_DEBUG);
-}
-
-void RKMDIWindowHistory::initActions (KActionCollection *ac, const char *prev_id, const char *next_id) {
-	RK_TRACE (APP);
-
-	prev_action = (KAction*) ac->addAction (prev_id, this, SLOT (prev()));
-	prev_action->setText (i18n ("Previous Window"));
-	prev_action->setIcon (QIcon (RKCommonFunctions::getRKWardDataDir () + "icons/window_back.png"));
-	prev_action->setShortcut (KShortcut (Qt::AltModifier + Qt::Key_Less, Qt::AltModifier + Qt::Key_Comma));
-
-	next_action = (KAction*) ac->addAction (next_id, this, SLOT (next()));
-	next_action->setText (i18n ("Next Window"));
-	next_action->setIcon (QIcon (RKCommonFunctions::getRKWardDataDir () + "icons/window_forward.png"));
-	next_action->setShortcut (KShortcut (Qt::AltModifier + Qt::Key_Greater, Qt::AltModifier + Qt::Key_Period));
-
-	updateActions ();
+	RK_DO (qDebug ("Remaining windows in history: %d", recent_windows.count ()), APP, DL_DEBUG);
 }
 
 void RKMDIWindowHistory::windowActivated (RKMDIWindow *window) {
 	RK_TRACE (APP);
 
 	if (!window) return;
-	if (window == current) return;
-	if (window->isToolWindow ()) return;		// exclude tool windows for now. Make configurable?
+	if (!recent_windows.isEmpty () && (window == recent_windows.last ())) return;
 
 	// update lists
-	back_list.removeAll (window);		// remove dupes
-	forward_list.clear ();
-	if (current) back_list.append (current);
-	current = window;
+	recent_windows.removeAll (window);		// remove dupes
+	recent_windows.append (window);
 
-	updateActions ();
+	updateSwitcher ();
 }
 
-void RKMDIWindowHistory::next () {
+void RKMDIWindowHistory::next (KAction* prev_action, KAction *next_action) {
 	RK_TRACE (APP);
 
-	if (!haveNext ()) return;
-	if (current) back_list.append (current);
-	current = forward_list.first ();
-	forward_list.pop_front ();
-
-	updateActions ();
-
-	RK_ASSERT (current);
-	current->activate ();
+	if (recent_windows.isEmpty ()) return;
+	getSwitcher (prev_action, next_action)->next ();
 }
 
-void RKMDIWindowHistory::prev () {
+void RKMDIWindowHistory::prev (KAction* prev_action, KAction *next_action) {
 	RK_TRACE (APP);
 
-	if (!havePrev ()) return;
-	if (current) forward_list.push_front (current);
-	current = back_list.last ();
-	back_list.pop_back ();
-
-	updateActions ();
-
-	RK_ASSERT (current);
-	current->activate ();
+	if (recent_windows.isEmpty ()) return;
+	getSwitcher (prev_action, next_action)->prev ();
 }
 
-bool RKMDIWindowHistory::haveNext () {
+RKMDIWindow* RKMDIWindowHistory::previousDocumentWindow () {
 	RK_TRACE (APP);
 
-	return (!forward_list.isEmpty ());
-}
-
-bool RKMDIWindowHistory::havePrev () {
-	RK_TRACE (APP);
-
-	return (!back_list.isEmpty ());
-}
-
-void RKMDIWindowHistory::updateActions () {
-	RK_TRACE (APP);
-
-	if (next_action) {
-		next_action->setEnabled (haveNext ());
-	}
-
-	if (prev_action) {
-		prev_action->setEnabled (havePrev ());
+	for (int i = recent_windows.count () - 1; i >= 0; --i) {
+		if (!recent_windows[i]->isToolWindow ()) return (recent_windows[i]);
 	}
+	return 0;
 }
 
-void RKMDIWindowHistory::windowDestroyed (QObject *window) {
+void RKMDIWindowHistory::updateSwitcher () {
 	RK_TRACE (APP);
 
-	back_list.removeAll (static_cast<RKMDIWindow *> (window));
-	forward_list.removeAll (static_cast<RKMDIWindow *> (window));
-	if (current == window) current = 0;
-	updateActions ();
+	if (switcher) switcher->update (recent_windows);
+}
+
+void RKMDIWindowHistory::removeWindow (RKMDIWindow *window) {
+	RK_TRACE (APP);
+
+	recent_windows.removeAll (window);
+	updateSwitcher ();
+}
+
+RKMDIWindowHistoryWidget* RKMDIWindowHistory::getSwitcher (KAction* prev_action, KAction *next_action) {
+	RK_TRACE (APP);
+
+	if (switcher) return switcher;
+
+	switcher = new RKMDIWindowHistoryWidget ();
+	connect (switcher, SIGNAL (destroyed(QObject*)), this, SLOT (switcherDestroyed()));
+	switcher->addAction (prev_action);
+	switcher->addAction (next_action);
+	switcher->update (recent_windows);
+	switcher->show ();
+	QWidget *act = QApplication::activeWindow ();
+	if (act) {
+		int center_x = act->x () + act->width () / 2;
+		int center_y = act->y () + act->height () / 2;
+		switcher->move (center_x - switcher->width () / 2, center_y - switcher->height () / 2);
+	} else {
+		RK_ASSERT (false);
+	}
+	switcher->setFocus ();
+
+	return switcher;
+}
+
+void RKMDIWindowHistory::switcherDestroyed () {
+	RK_TRACE (APP);
+
+	RK_ASSERT (switcher);
+	switcher = 0;
+}
+
+void RKMDIWindowHistory::popLastWindow (RKMDIWindow* match) {
+	RK_TRACE (APP);
+
+	if (recent_windows.isEmpty ()) return;
+	else if (recent_windows.last () == match) recent_windows.removeLast ();
+	updateSwitcher ();
 }
 
 #include "rkworkplace.moc"
