@@ -2,7 +2,7 @@
                           rkworkplace  -  description
                              -------------------
     begin                : Thu Sep 21 2006
-    copyright            : (C) 2006, 2007, 2009, 2010 by Thomas Friedrichsmeier
+    copyright            : (C) 2006, 2007, 2009, 2010, 2011 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -24,10 +24,9 @@
 #include <QSplitter>
 
 #include <kurl.h>
-#include <kmultitabbar.h>
 
 #include "rkmdiwindow.h"
-#include "../rbackend/rcommandreceiver.h"
+#include "rktoolwindowlist.h"
 
 class RObject;
 class RCommandChain;
@@ -36,6 +35,9 @@ class RKEditor;
 class KActionCollection;
 class KAction;
 class RKToolWindowBar;
+class RKMDIWindowHistoryWidget;
+
+#define TOOL_WINDOW_BAR_COUNT 4
 
 /** Simple class to store the history of recently used RKMDIWindow */
 class RKMDIWindowHistory : public QObject {
@@ -44,34 +46,33 @@ public:
 	RKMDIWindowHistory (QObject *parent);
 	~RKMDIWindowHistory ();
 
-	void initActions (KActionCollection *ac, const char *prev_id, const char *next_id);
-	bool haveNext ();
-	bool havePrev ();
-	void windowDestroyed (QObject *window);
+	void removeWindow (RKMDIWindow *window);
+/** pops the last window from the list, if it matches the given pointer */
+	void popLastWindow (RKMDIWindow *match);
+	RKMDIWindow *previousDocumentWindow ();
+	void next (KAction *prev_action, KAction *next_action);
+	void prev (KAction *prev_action, KAction *next_action);
 public slots:
-	void next ();
-	void prev ();
 	void windowActivated (RKMDIWindow *window);
+private slots:
+	void switcherDestroyed ();
 private:
-	void updateActions ();
-
-	RKMDIWindow *current;
-	QList<RKMDIWindow *> back_list;
-	QList<RKMDIWindow *> forward_list;
-	KAction *next_action;
-	KAction *prev_action;
+	void updateSwitcher ();
+	QList<RKMDIWindow *> recent_windows;
+	RKMDIWindowHistoryWidget *switcher;
+	RKMDIWindowHistoryWidget *getSwitcher (KAction *prev_action, KAction *next_action);
 };
 
 /** This class (only one instance will probably be around) keeps track of which windows are opened in the workplace, which are detached, etc. Also it is responsible for creating and manipulating those windows.
 It also provides a QWidget (RKWorkplace::view ()), which actually manages the document windows (only those, so far. I.e. this is a half-replacement for KMdi, which will be gone in KDE 4). Currently layout of the document windows is always tabbed. */
-class RKWorkplace : public QWidget, public RCommandReceiver {
+class RKWorkplace : public QWidget {
 	Q_OBJECT
 public:
 /** ctor.
 @param parent: The parent widget for the workspace view (see view ()) */
 	explicit RKWorkplace (QWidget *parent);
 	~RKWorkplace ();
-	void initActions (KActionCollection *ac, const char *prev_id, const char *next_id, const char *left_id, const char *right_id);
+	void initActions (KActionCollection *ac, const char *left_id, const char *right_id);
 
 /** @returns a pointer to the view of the workplace. Since possibly the workplace layout might change, better not rely on this pointer being valid for long */
 	RKWorkplaceView *view () { return wview; };
@@ -100,14 +101,14 @@ public:
 @param read_only Open the document read only? Default is false, i.e. Read-write
 @param force_caption Usually the caption is determined from the url of the file. If you specify a non-empty string here, that is used instead.
 @returns false if a local url could not be opened, true for all remote urls, and on success */
-	bool openScriptEditor (const KUrl &url=KUrl (), const QString& encoding=QString (), bool use_r_highlighting=true, bool read_only=false, const QString &force_caption = QString::null);
+	RKMDIWindow* openScriptEditor (const KUrl &url=KUrl (), const QString& encoding=QString (), bool use_r_highlighting=true, bool read_only=false, const QString &force_caption = QString::null, bool delete_on_close=false);
 /** Opens a new help window, starting at the given url
 @param url URL to open
 @param only_once if true, checks whether any help window already shows this URL. If so, raise it, but do not open a new window. Else show the new window */
-	void openHelpWindow (const KUrl &url=KUrl (), bool only_once=false);
+	RKMDIWindow* openHelpWindow (const KUrl &url=KUrl (), bool only_once=false);
 /** Opens a new output window. Currently only a single output window will ever be created. Subsequent calls to the function will not create additional windows right now (but will raise / refresh the output window
 @param url currently ignored! */
-	void openOutputWindow (const KUrl &url=KUrl ());
+	RKMDIWindow* openOutputWindow (const KUrl &url=KUrl ());
 
 	void newX11Window (WId window_to_embed, int device_number);
 	void newObjectViewer (RObject *object);
@@ -145,28 +146,22 @@ Has no effect, if RKSettingsModuleGeneral::workplaceSaveMode () != RKSettingsMod
 	void restoreWorkplace (RCommandChain *chain=0);
 /** Like the other restoreWorkplace (), but takes the description as a parameter rather than reading from the R workspace. To be used, when RKSettingsModuleGeneral::workplaceSaveMode () == RKSettingsModuleGeneral::SaveWorkplaceWithSeesion
 @param description workplace description */
-	void restoreWorkplace (const QString &description);
-/** Clear the description as set by saveWorkplace () from the R backend. Simply removes the internal object. Since the description is only needed while the workplace is being saved / restored, this should be called shortly after saveWorkplace () and restoreWorkplace ()
-Has no effect, if RKSettingsModuleGeneral::workplaceSaveMode () != RKSettingsModuleGeneral::SaveWorkplaceWithWorkspace
-@param cahin command chain to place the command in */
-	void clearWorkplaceDescription (RCommandChain *chain=0);
+	void restoreWorkplace (const QStringList &description);
 
-	QString makeWorkplaceDescription (const QString &sep, bool quote);
+	QStringList makeWorkplaceDescription ();
 
 /** In the current design there is only ever one workplace. Use this static function to reference it.
 @returns a pointer to the workplace */
 	static RKWorkplace *mainWorkplace () { return main_workplace; };
-	void placeInToolWindowBar (RKMDIWindow *window, KMultiTabBar::KMultiTabBarPosition position);
+	static RKMDIWindowHistory *getHistory () { return main_workplace->history; };
+	void placeToolWindows ();
 signals:
 /** TODO: For future expansion. This signal is neither emitted nor used so far. It could be used to deactivate some options in the "Window" menu. Or maybe it can be removed? */
 	void lastWindowClosed ();
 public slots:
 /** When windows are attached to the workplace, their QObject::destroyed () signal is connected to this slot. Thereby deleted objects are removed from the workplace automatically */
-	void windowDestroyed (QObject *window);
+	void removeWindow (QObject *window);
 	void saveSettings ();
-protected:
-/** handles the result of the command issued in restoreWorkplace. */
-	void rCommandDone (RCommand *command);
 private:
 /** current list of windows. @See getObjectList () */ 
 	RKWorkplaceObjectList windows;
@@ -179,13 +174,14 @@ private:
 /** a window was removed. Try to activate some other window. */
 	void windowRemoved ();
 
-	void restoreWorkplaceItem (const QString &desc);
 	RKMDIWindowHistory *history;
 
 	QSplitter *horiz_splitter;
 	QSplitter *vert_splitter;
 
-	RKToolWindowBar* tool_window_bars[4];
+	RKToolWindowBar* tool_window_bars[TOOL_WINDOW_BAR_COUNT];
+friend class RKToolWindowBar;
+	void placeInToolWindowBar (RKMDIWindow *window, int position);
 };
 
 #endif
