@@ -3,7 +3,7 @@ local({
 ## Compute
 ## Print result
 rk.header ("Hodrick-Prescott Filter", parameters=list("Lambda", 1600))
-x <- get("co2", envir=globalenv())
+x <- co2
 lambda <- 1600
 
 if (any (is.na (x))) stop ("Missing values cannot be handled")
@@ -15,8 +15,8 @@ if (is.ts(x)) {
 	trend <- ts(trend,start(x),frequency=frequency(x))
 	cycle <- ts(cycle,start(x),frequency=frequency(x))
 }
-assign("hptrend", trend, envir=globalenv())
-assign("hpcycle", cycle, envir=globalenv())
+.GlobalEnv$hptrend <- trend
+.GlobalEnv$hpcycle <- cycle
 rk.graph.on ()
 try({
 	par(mfrow=c(2,1),mar=c(2,4,2,2)+0.1)
@@ -25,5 +25,5 @@ try({
 })
 rk.graph.off ()
 })
-.rk.rerun.plugin.link(plugin="rkward::hp_filter", settings="create_cycle.state=1\ncreate_trend.state=1\ncustom.state=0\ncycle_col.color.string=green4\ncycle_lty.string=\ncycle_lwd.real=1.00\ncycle_name.selection=hpcycle\ndownlab.text=\nlambda.string=1600\nplot_cycle.state=1\nseries_col.color.string=blue\nseries_lty.string=\nseries_lwd.real=1.00\ntrend_col.color.string=red\ntrend_lty.string=\ntrend_lwd.real=1.00\ntrend_name.selection=hptrend\nuplab.text=\nx.available=co2", label="Run again")
+.rk.rerun.plugin.link(plugin="rkward::hp_filter", settings="custom.state=0\ncycle_col.color.string=green4\ncycle_lty.string=\ncycle_lwd.real=1.00\ncycle_name.active=1\ncycle_name.objectname=hpcycle\ncycle_name.parent=.GlobalEnv\ndownlab.text=\nlambda.string=1600\nplot_cycle.state=1\nseries_col.color.string=blue\nseries_lty.string=\nseries_lwd.real=1.00\ntrend_col.color.string=red\ntrend_lty.string=\ntrend_lwd.real=1.00\ntrend_name.active=1\ntrend_name.objectname=hptrend\ntrend_name.parent=.GlobalEnv\nuplab.text=\nx.available=co2", label="Run again")
 .rk.make.hr()
