@@ -11,7 +11,7 @@ rk.header ("Coefficients:", level=4)
 rk.print (coef(estimates.2pl))
 rk.print (paste("Log-likelihood value at convergence:",round(estimates.2pl$log.Lik, digits=1)))
 # keep results in current workspace
-estimates.2pl <<- estimates.2pl
+.GlobalEnv$estimates.2pl <- estimates.2pl
 })
-.rk.rerun.plugin.link(plugin="rkward::par_est_2pl", settings="chk_save.state=save\nconstraint.available=\nghk_2pl.real=15.00\ninteract.state=TRUE\nirtparam.state=TRUE\niterem.real=40.00\niterqn_2pl.real=150.00\nnaaction.state=\noptimeth.string=BFGS\nsave_name.selection=estimates.2pl\nstartval.string=NULL\nverbose.state=\nx.available=WIRS", label="Run again")
+.rk.rerun.plugin.link(plugin="rkward::par_est_2pl", settings="constraint.available=\nghk_2pl.real=15.00\ninteract.state=TRUE\nirtparam.state=TRUE\niterem.real=40.00\niterqn_2pl.real=150.00\nnaaction.state=\noptimeth.string=BFGS\nsave_name.active=1\nsave_name.objectname=estimates.2pl\nsave_name.parent=.GlobalEnv\nstartval.string=NULL\nverbose.state=\nx.available=WIRS", label="Run again")
 .rk.make.hr()
