@@ -1,9 +1,3 @@
-## intro
-# This should be the first line in each test suite file: Include the
-# test framework, unless already included (multiple inclusion would not
-# really do any harm either, though
-if (!isClass ("RKTestSuite")) source ("test_framework.R")
-
 ## definition of the test suite
 suite <- new ("RKTestSuite", id="rkward_application_tests",
 	# place here libraries that are required for *all* tests in this suite, or highly likely to be installed
@@ -47,10 +41,17 @@ suite <- new ("RKTestSuite", id="rkward_application_tests",
 		}),
 		new ("RKTest", id="output_graphics_formats", call=function () {
 			rk.graph.on(); plot (1, 1); rk.graph.off()	# should produce PNG, 480*480
-
-			options (rk.graphics.type="JPG", rk.graphics.width=500, rk.graphics.height=500, rk.graphics.jpg.quality=34)
+			old.options <- options()
+			options (rk.graphics.type="JPG",
+				 rk.graphics.width=500,
+				 rk.graphics.height=500,
+				 rk.graphics.jpg.quality=34)
 			rk.graph.on(); plot (1, 1); rk.graph.off()
-			rktest.initializeEnvironment ()	# restore options
+			# restore options
+			options (rk.graphics.type=old.options$rk.graphics.type,
+				 rk.graphics.width=old.options$rk.graphics.width,
+				 rk.graphics.height=old.options$rk.graphics.height,
+				 rk.graphics.jpg.quality=old.options$rk.graphics.jpg.quality)
 
 			rk.graph.on (device.type="SVG", width=300); plot (1, 1); rk.graph.off ()
 		}),
@@ -60,10 +61,10 @@ suite <- new ("RKTestSuite", id="rkward_application_tests",
 			#
 			# In R 2.11.0, str (x) produces an error "subecript out of bounds" (only if library (XML) is loaded!
 			# The main concern is that we should handle this object gracefully, i.e. do not crash while syncing it.
-			load ("../rkward_application_tests_strange_object.RData")
+			load ("rkward_application_tests_strange_object.RData")
 			rk.sync.global ()
 			rk.sync (x)
-		}, libraries=c ("XML")),
+		}, libraries=c ("XML"), files=c("../rkward_application_tests_strange_object.RData")),
 		new ("RKTest", id="dev_off_bug", call=function () {
 			graphics.off()
 			stopifnot (is.null (dev.list ()))
@@ -256,7 +257,3 @@ suite <- new ("RKTestSuite", id="rkward_application_tests",
 		}
 	)
 )
-
-## always store the result in "results" and print it
-results <- rktest.runRKTestSuite (suite)
-print (results)
